from Types cimport *
from MultiplexDeltaMasses cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/MultiplexIsotopicPeakPattern.h>" namespace "OpenMS":
    
    cdef cppclass MultiplexIsotopicPeakPattern "OpenMS::MultiplexIsotopicPeakPattern":

        MultiplexIsotopicPeakPattern(MultiplexIsotopicPeakPattern) nogil except + #wrap-ignore
        MultiplexIsotopicPeakPattern(int c, int ppp, MultiplexDeltaMasses ms, int msi) nogil except +

        int getCharge() nogil except +
        int getPeaksPerPeptide() nogil except +
        MultiplexDeltaMasses getMassShifts() nogil except +
        int getMassShiftIndex() nogil except +
        unsigned getMassShiftCount() nogil except +
        double getMassShiftAt(int i) nogil except +
        double getMZShiftAt(int i) nogil except +
        unsigned getMZShiftCount() nogil except +




from libcpp.vector cimport vector as libcpp_vector
from String cimport *

from ProteinIdentification cimport *
from PeptideIdentification cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *

from libcpp cimport bool

cdef extern from "<OpenMS/FORMAT/ProtXMLFile.h>" namespace "OpenMS":

    cdef cppclass ProtXMLFile:

        ProtXMLFile() nogil except +
        ProtXMLFile(ProtXMLFile) nogil except + # wrap-ignore

        void load(String filename, ProteinIdentification & protein_ids, PeptideIdentification & peptide_ids) nogil except +

        # Not implemented
        void store(String filename, ProteinIdentification & protein_ids, PeptideIdentification & peptide_ids, String document_id) nogil except +




from Peak1D cimport *
from Feature cimport *
from FeatureMap cimport *
from MSExperiment cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *
from FeatureFinder cimport *
from DefaultParamHandler cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/FeatureFinderAlgorithmPicked.h>" namespace "OpenMS":

    cdef cppclass FeatureFinderAlgorithmPicked(DefaultParamHandler):

        # wrap-inherits:
        #    DefaultParamHandler
        FeatureFinderAlgorithmPicked()      nogil except +

        void setData(MSExperiment & input, FeatureMap & output, FeatureFinder & ff) nogil except +
        void run() nogil except +


        void setSeeds(FeatureMap& seeds) nogil except +

        # static FeatureFinderAlgorithm* create()

#
# static methods are wrapped like this:
#

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/FeatureFinderAlgorithmPicked.h>" namespace "OpenMS::FeatureFinderAlgorithmPicked":

    String getProductName()   nogil except + # wrap-attach:FeatureFinderAlgorithmPicked



from Types cimport *
from BaseFeature cimport *
from MSExperiment cimport *
from KDTreeFeatureMaps cimport *
from libcpp.map cimport map as libcpp_map
from libcpp.vector cimport vector as libcpp_vector



cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/FeatureMapping.h>" namespace "OpenMS":

    cdef cppclass FeatureMapping "OpenMS::FeatureMapping":
       FeatureMapping() nogil except +
       FeatureMapping(FeatureMapping) nogil except +

       FeatureMapping_FeatureToMs2Indices assignMS2IndexToFeature(MSExperiment& spectra,
                                                                  KDTreeFeatureMaps& fp_map_kd,
                                                                  double& precursor_mz_tolerance,
                                                                  double& precursor_rt_tolerance,
                                                                  bool ppm) nogil except +

    cdef cppclass FeatureMapping_FeatureToMs2Indices "OpenMS::FeatureMapping::FeatureToMs2Indices":
        FeatureMapping_FeatureToMs2Indices()
        FeatureMapping_FeatureToMs2Indices(FeatureMapping_FeatureToMs2Indices)









from Types cimport *
from DefaultParamHandler cimport *
from ConsensusMap cimport *
from IsobaricQuantitationMethod cimport *
from ItraqFourPlexQuantitationMethod cimport *
from ItraqEightPlexQuantitationMethod cimport *
from TMTSixPlexQuantitationMethod cimport *
from TMTTenPlexQuantitationMethod cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/IsobaricQuantifier.h>" namespace "OpenMS":
    
    cdef cppclass IsobaricQuantifier(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        IsobaricQuantifier(IsobaricQuantifier) nogil except +

        IsobaricQuantifier(IsobaricQuantitationMethod *quant_method) nogil except +  # wrap-ignore
        IsobaricQuantifier(ItraqFourPlexQuantitationMethod *quant_method) nogil except +
        IsobaricQuantifier(ItraqEightPlexQuantitationMethod *quant_method) nogil except +
        IsobaricQuantifier(TMTSixPlexQuantitationMethod *quant_method) nogil except +
        IsobaricQuantifier(TMTTenPlexQuantitationMethod *quant_method) nogil except +

        void quantify(ConsensusMap & consensus_map_in, ConsensusMap & consensus_map_out) nogil except +




# input-encoding: latin-1
from __future__ import print_function

# windows ?
import sys
iswin = sys.platform == "win32"

# osx ?
isosx = sys.platform == "darwin"
if isosx:
    import platform
    osx_ver = platform.mac_ver()[0] #e.g. ('10.15.1', ('', '', ''), 'x86_64')

import sys
single_threaded = False
no_optimization = False
if "--single-threaded" in sys.argv:
    single_threaded = True
    sys.argv.remove("--single-threaded")
if "--no-optimization" in sys.argv:
    no_optimization = True
    sys.argv.remove("--no-optimization")

# import config
from env import  (OPEN_MS_COMPILER, OPEN_MS_SRC, OPEN_MS_GIT_BRANCH, OPEN_MS_BUILD_DIR, OPEN_MS_CONTRIB_BUILD_DIRS,
                  QT_INSTALL_LIBS, QT_INSTALL_BINS, MSVS_RTLIBS,
                  OPEN_MS_BUILD_TYPE, OPEN_MS_VERSION, LIBRARIES_EXTEND,
                  LIBRARY_DIRS_EXTEND, OPEN_MS_LIB, OPEN_SWATH_ALGO_LIB, PYOPENMS_INCLUDE_DIRS,
                  PY_NUM_MODULES, PY_NUM_THREADS, SYSROOT_OSX_PATH, LIBRARIES_TO_BE_PARSED_EXTEND, OPENMS_GIT_LC_DATE_FORMAT)

IS_DEBUG = OPEN_MS_BUILD_TYPE.upper() == "DEBUG"

if iswin and IS_DEBUG:
    raise Exception("building pyopenms on windows in debug mode not tested yet.")

# use autowrap to generate Cython and .cpp file for wrapping OpenMS:
import pickle
import os
import glob
import re
import shutil
import time

if OPEN_MS_GIT_BRANCH == "nightly":
    package_name = "pyopenms_nightly"
    package_version = OPEN_MS_VERSION + ".dev" + OPENMS_GIT_LC_DATE_FORMAT
else:
    package_name = "pyopenms"
    package_version = OPEN_MS_VERSION

os.environ["CC"] = OPEN_MS_COMPILER
# AFAIK distutils does not care about CXX (set it to be safe)
os.environ["CXX"] = OPEN_MS_COMPILER

j = os.path.join

src_pyopenms = j(OPEN_MS_SRC, "src/pyOpenMS")
extra_includes = glob.glob(src_pyopenms + "/extra_includes/*.h*")

for include in extra_includes:
    shutil.copy(include, "extra_includes/")


persisted_data_path = "include_dir.bin"
autowrap_include_dirs = pickle.load(open(persisted_data_path, "rb"))

# patch for parallel compilation
# https://stackoverflow.com/questions/11013851/speeding-up-build-process-with-distutils
# -- this is not what we want, we dont want to compile each object with
#    multiple threads, we want to compile multiple extensions at the same time:
from setuptools import setup, Extension
import multiprocessing.pool
def parallel_build_extensions(self):
    # taken from distutils/command/build_ext.py
    # see also Cython/Distutils/old_build_ext.py
    #  - note that we are missing the self.cython_sources line, so this will not work under all circumstances
    # First, sanity-check the 'extensions' list
    self.check_extensions_list(self.extensions)
    mypool = multiprocessing.pool.ThreadPool(int(PY_NUM_THREADS))
    list(mypool.imap(self.build_extension, self.extensions))
if not single_threaded:
    import distutils.command.build_ext
    distutils.command.build_ext.build_ext.build_extensions = parallel_build_extensions
    import Cython.Distutils.build_ext
    distutils.command.build_ext.build_ext.build_extensions = parallel_build_extensions

with open("pyopenms/version.py", "w") as fp:
    print("version=%r" % package_version, file=fp)

# parse config

if OPEN_MS_CONTRIB_BUILD_DIRS.endswith(";"):
    OPEN_MS_CONTRIB_BUILD_DIRS = OPEN_MS_CONTRIB_BUILD_DIRS[:-1]

for OPEN_MS_CONTRIB_BUILD_DIR in OPEN_MS_CONTRIB_BUILD_DIRS.split(";"):
    if os.path.exists(os.path.join(OPEN_MS_CONTRIB_BUILD_DIR, "lib")):
        break


# Package data expected to be installed. On Linux the debian package
# contains share/ data and must be installed to get access to the OpenMS shared
# library.
#
if iswin:
    if IS_DEBUG:
        libraries = ["OpenMSd", "OpenSwathAlgod", "SuperHirnd", "Qt5Cored", "Qt5Networkd"]
    else:
        libraries = ["OpenMS", "OpenSwathAlgo", "SuperHirn", "Qt5Core", "Qt5Network"]
elif sys.platform.startswith("linux"):
    libraries = ["OpenMS", "OpenSwathAlgo", "SuperHirn", "Qt5Core", "Qt5Network"]
elif sys.platform == "darwin":
    libraries = ["OpenMS", "OpenSwathAlgo", "SuperHirn"]
else:
    print("\n")
    print("platform", sys.platform, "not supported yet")
    print("\n")
    exit()

if (iswin):
    library_dirs = [OPEN_MS_BUILD_DIR,
                    j(OPEN_MS_BUILD_DIR, "lib", "Release"),
                    j(OPEN_MS_BUILD_DIR, "bin", "Release"),
                    j(OPEN_MS_BUILD_DIR, "Release"),
                    QT_INSTALL_BINS,
                    QT_INSTALL_LIBS,
                    ]
else:
    library_dirs = [OPEN_MS_BUILD_DIR,
                j(OPEN_MS_BUILD_DIR, "lib"),
                j(OPEN_MS_BUILD_DIR, "bin"),
                QT_INSTALL_BINS,
                QT_INSTALL_LIBS,
                ]

# extend with contrib lib dirs
for OPEN_MS_CONTRIB_BUILD_DIR in OPEN_MS_CONTRIB_BUILD_DIRS.split(";"):
  library_dirs.append(j(OPEN_MS_CONTRIB_BUILD_DIR, "lib"))

import numpy

include_dirs = [
    "extra_includes",
    j(numpy.core.__path__[0], "include"),
]

# append all include and library dirs exported by CMake
include_dirs.extend(PYOPENMS_INCLUDE_DIRS.split(";"))
library_dirs.extend(LIBRARY_DIRS_EXTEND)
libraries.extend(LIBRARIES_EXTEND)


# libraries of any type to be parsed and added
objects = []
add_libs = LIBRARIES_TO_BE_PARSED_EXTEND.split(";")
for lib in add_libs:
  if not iswin:
    if lib.endswith(".a"):
      objects.append(lib)
      name_search = re.search('.*/lib(.*)\.a$', lib)
      if name_search:
        libraries.append(name_search.group(1))
        library_dirs.append(os.path.dirname(lib))
    if lib.endswith(".so") or lib.endswith(".dylib"):
      name_search = re.search('.*/lib(.*)\.(so|dylib)$', lib)
      if name_search:
        libraries.append(name_search.group(1))
        library_dirs.append(os.path.dirname(lib))
  else:
    if lib.endswith(".lib"):
      name_search = re.search('.*/(.*)\.lib$', lib)
      if name_search:
        libraries.append(name_search.group(1))
        library_dirs.append(os.path.dirname(lib))


extra_link_args = []
extra_compile_args = []

if iswin:
    # /EHs is important. It sets _CPPUNWIND which causes boost to
    # set BOOST_NO_EXCEPTION in <boost/config/compiler/visualc.hpp>
    # such that  boost::throw_excption() is declared but not implemented.
    # The linker does not like that very much ...
    extra_compile_args = ["/EHs", "/bigobj"]
elif sys.platform.startswith("linux"):
    extra_link_args = ["-Wl,-s"]
elif sys.platform == "darwin":
    library_dirs.insert(0,j(OPEN_MS_BUILD_DIR,"pyOpenMS","pyopenms"))
    # we need to manually link to the Qt Frameworks
    extra_compile_args = ["-Qunused-arguments"]
    extra_link_args = ["-Wl,-rpath","-Wl,@loader_path/"]
if IS_DEBUG:
    extra_compile_args.append("-g2")

# Note: we use -std=gnu++11 in Linux by default, also reduce some warnings
if not iswin:
    extra_link_args.append("-std=c++11")
    if isosx: # MacOS c++11
        extra_link_args.append("-stdlib=libc++") # MacOS libstdc++ does not include c++11 lib support.
        extra_link_args.append("-mmacosx-version-min=10.7") # due to libc++
    extra_compile_args.append("-std=c++11")
    if isosx: # MacOS c++11
        extra_compile_args.append("-stdlib=libc++")
        extra_compile_args.append("-mmacosx-version-min=10.7")
        if (osx_ver >= "10.14.0" and SYSROOT_OSX_PATH): # since macOS Mojave
            extra_compile_args.append("-isysroot" + SYSROOT_OSX_PATH)
    extra_compile_args.append("-Wno-redeclared-class-member")
    extra_compile_args.append("-Wno-unused-local-typedefs")
    extra_compile_args.append("-Wno-deprecated-register")
    extra_compile_args.append("-Wdeprecated-declarations")
    extra_compile_args.append("-Wno-sign-compare")
    extra_compile_args.append("-Wno-unknown-pragmas")
    extra_compile_args.append("-Wno-header-guard")
    extra_compile_args.append("-Wno-unused-function")
    extra_compile_args.append("-Wno-deprecated-declarations")
    extra_compile_args.append("-Wno-missing-declarations")
    if no_optimization:
        extra_compile_args.append("-O0")
        extra_link_args.append("-O0")

mnames = ["pyopenms_%s" % (k+1) for k in range(int(PY_NUM_MODULES))]
ext = []
for module in mnames:

    ext.append(Extension(
        module,
        sources=["pyopenms/%s.cpp" % module],
        language="c++",
        library_dirs=library_dirs,
        libraries=libraries,
        include_dirs=include_dirs + autowrap_include_dirs,
        extra_compile_args=extra_compile_args,
        extra_objects=objects,
        extra_link_args=extra_link_args,
		define_macros=[('BOOST_ALL_NO_LIB', None)] ## Deactivates boost autolink (esp. on win).
		## Alternative is to specify the boost naming scheme (--layout param; easy if built from contrib)
		## TODO just take over compile definitions from OpenMS (CMake)
    ))

share_data = []
share_data.append("License.txt")

# enforce 64bit-only build as OpenMS is not available in 32bit on osx
if sys.platform == "darwin":
    os.environ['ARCHFLAGS'] = "-arch x86_64"

setup(

    name=package_name,
    packages=["pyopenms"],
    ext_package="pyopenms",
	install_requires=[
          'numpy',
    ],

    version=package_version,

    maintainer="Uwe Schmitt",
    maintainer_email="uschmitt@mineway.de",
    license="http://opensource.org/licenses/BSD-3-Clause",
    platforms=["any"],
    description="Python wrapper for C++ LCMS library OpenMS",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: BSD License",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Topic :: Scientific/Engineering :: Chemistry",
    ],
    long_description=open("README.rst").read(),
    zip_safe=False,

    url="http://open-ms.de",

    author="Uwe Schmitt",
    author_email="uschmitt@mineway.de",

    ext_modules=ext,
    include_package_data=True  # see MANIFEST.in
)



from libcpp.vector cimport vector as libcpp_vector
from DefaultParamHandler cimport *
from PeptideIdentification cimport *
from MSSpectrum cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *

cdef extern from "<OpenMS/ANALYSIS/DENOVO/DeNovoIdentification.h>" namespace "OpenMS":
    
    cdef cppclass DeNovoIdentification "OpenMS::DeNovoIdentification":
        # wrap-ignore
        # ABSTRACT CLASS
        # no-pxd-import
        DeNovoIdentification() nogil except +
        DeNovoIdentification(DeNovoIdentification) nogil except +
        void getIdentifications(libcpp_vector[ PeptideIdentification ] & ids, MSExperiment & exp) nogil except +
        void getIdentification(PeptideIdentification & id_, MSSpectrum & spectrum) nogil except +



# --------------------------------------------------------------------------
#                   OpenMS -- Open-Source Mass Spectrometry
# --------------------------------------------------------------------------
# Copyright The OpenMS Team -- Eberhard Karls University Tuebingen,
# ETH Zurich, and Freie Universitaet Berlin 2002-2020.
#
# This software is released under a three-clause BSD license:
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of any author or any participating institution
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
# For a full list of authors, refer to the file AUTHORS.
# --------------------------------------------------------------------------
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL ANY OF THE AUTHORS OR THE CONTRIBUTING
# INSTITUTIONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# --------------------------------------------------------------------------
# $Maintainer: Chris Bielow $
# $Authors: Chris Bielow $
# --------------------------------------------------------------------------

import re
import random
import math
import sys
import argparse

## holds FASTA header + sequence
class FASTAEntry:
    pass

## grab entries from FASTA file
def nextEntry(fileobj):
        entry = FASTAEntry()
        entry.header = fileobj.readline()
        entry.sequence = ""
        for line in fileobj:
                if '>' == line[0]:
                        yield entry
                        entry.header = line
                        entry.sequence = ""
                else:  
                        entry.sequence += line
        yield entry


## sample abundance from Gaussian in log space
def sampleAbundance(mu=3, sigma=1):
    return math.exp(random.gauss(mu, sigma))


def main(argv):
    ## we use ArgumentParser, which requires 2.7
    if sys.version_info < (2, 7):
      raise "This script requires python 2.7 or greater"

    ## add weight filtering functionality if BioPython is available
    try:
      from Bio.SeqUtils.ProtParam import ProteinAnalysis
      has_biopython = 1
    except :
      has_biopython = 0
      
      
    parser = argparse.ArgumentParser(description='Add abundance to FASTA files.')
    parser.add_argument('infile', type=argparse.FileType('r'), help='Input FASTA file')
    parser.add_argument('outfile', type=argparse.FileType('w'), help='Output FASTA file')
    
    parser.add_argument('--mu', dest='mu', action='store', default=3, help='mean of gaussian in log space')
    parser.add_argument('--sigma', dest='sigma', action='store', default=1, help='sd of gaussian in log space')
    parser.add_argument('--sample', dest='sample', action='store', default=0, help='Number of entries to keep (for sampling a bigger FASTA file)')
    parser.add_argument('--random', dest='random', action='store_true', help='Randomly shuffle entries before sampling (only if --sample is given). If not given, the first \'X\' samples are used.')
    if (has_biopython):
      parser.add_argument('--weight_low', dest='weight_low', action='store', default=0, help='minimum molecular weight of protein')
      parser.add_argument('--weight_up', dest='weight_up', action='store', default=0, help='Maximum molecular weight of protein (use 0 for unlimited)')
    else:
      print ("Warning: protein weight filtering not supported, as BioPython module is not installed.")
      
    ## argument parsing
    args = parser.parse_args()
    fileobj = args.infile
    fileoutobj = args.outfile
    sample_size = int(args.sample)
    sample_random = bool(args.random)
    if (has_biopython):
      weight_low = float(args.weight_low)
      weight_up = float(args.weight_up)
      if (weight_up <= 0): weight_up = sys.float_info.max
      
    
    ## list of final entries
    fasta_entries = []
    
    for entry in nextEntry(fileobj):
        header = entry.header
        ## check if it contains 'intensity'?
        rep = re.compile(r"\[# *(.*) *#\]")
        m = rep.search(header)
        header_new = ""
        other = []
        if (m):
          header_new = header.replace(m.group(0), "") ## delete meta
          for element in m.group(1).split(','):
              #print "element:", element
              if (element.find("intensity") == -1):
                  other.append(element)
        else:
          header_new = header  ## nothing to replace

        ## create new metainfo array
        i = "intensity=" + str(sampleAbundance(float(args.mu), float(args.sigma)))
        other.append(i)

        entry.header = header_new.rstrip() + "[# " + (", ").join(other) + " #]"
        
        if (has_biopython):
          sequence = "".join(entry.sequence.split("\n"))
          ##
          ## BioPython does not like some AA letters - they need replacement
          ##
          ## replace "U" (Selenocystein) with "C" (Cystein)
          sequence = sequence.replace("U","C")
          ## replace "X" (unknown) with "P" (Proline) [arbitrary choice - but weight of 115 is very close to averagine]
          sequence = sequence.replace("X","P")
          ## replace "B" (Asparagine or aspartic acid) with "N" (Asparagine) 
          sequence = sequence.replace("B","N")
          ## replace "Z" (Glutamine or glutamic acid) with "Q" (Glutamine) 
          sequence = sequence.replace("Z","Q")
          ## replace "Z" (Glutamine or glutamic acid) with "Q" (Glutamine) 
          sequence = sequence.replace("Z","Q")
          ## replace "J" (Leucine or Isoleucine) with "L" (Leucine) 
          sequence = sequence.replace("J","L")
          analysed_seq = ProteinAnalysis(sequence)
          weight = analysed_seq.molecular_weight()
          if (not(weight_low <= weight and weight <= weight_up)):
            continue
        
        
        fasta_entries.append(entry.header + "\n" + entry.sequence)
         
        ## only read to sample size (the rest is thrown away anyways)
        if (sample_size > 0 and not(sample_random)):
          if (len(fasta_entries) >= sample_size):
            break
          
        
    ## select subset (if required)
    if (sample_size > 0):
      indices = range(0,len(fasta_entries))
      ## random sampling only makes sense if we take a subset
      if (sample_random and sample_size < len(fasta_entries)):
        random.shuffle(indices)
      indices = [indices[i] for i in range(0,sample_size)]
      fasta_entries = [fasta_entries[i] for i in indices]
      
    ## write to file
    for entry in fasta_entries:
      fileoutobj.write(entry)

    print ("Generated " + str(len(fasta_entries)) + " protein sequences")

if __name__ == "__main__":
    main(sys.argv)



from Types cimport *
from libcpp.string cimport string as libcpp_string
from libcpp.vector cimport vector as libcpp_vector
from ISpectrumAccess cimport *

# ctypedef shared_ptr[ISpectrumAccess] SpectrumAccessPtr

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/DATAACCESS/SpectrumAccessTransforming.h>" namespace "OpenMS":
    
    cdef cppclass SpectrumAccessTransforming(ISpectrumAccess) :
        # wrap-inherits:
        #  ISpectrumAccess
        # wrap-ignore
        # ABSTRACT class

        SpectrumAccessTransforming(SpectrumAccessTransforming) nogil except + # wrap-ignore





import pyopenms
"""
Producing the test data for TOPP_FeatureLinkerUnlabeledQT_5 and TOPP_FeatureLinkerUnlabeledQT_6
"""

fmaps = [ pyopenms.FeatureMap() for i in range(3)]
pepids = []
pepseq = ["PEPTIDEA", "PEPTIDEK", "PEPTIDER"]
for s in pepseq:
    pepid = pyopenms.PeptideIdentification()
    hit = pyopenms.PeptideHit()
    hit.setSequence(pyopenms.AASequence.fromString(s, True))
    pepid.insertHit(hit)
    pepid.setIdentifier("Protein0")
    pepids.append(pepid)

protid = pyopenms.ProteinIdentification()
protid.setIdentifier("Protein0")
for i,fmap in enumerate(fmaps):
    fmap.setProteinIdentifications( [protid])
    # add 3 features to each map, but with a twist (adding different peptide ids to different maps)
    for k in range(3):
        f = pyopenms.Feature()
        f.setRT(300 + k*100 + i*10)
        f.setMZ(500 + k*0.001 + i*0.01)
        f.setIntensity(500 + i*100)
        f.setMetaValue("sequence", pepseq[ (i+k) % 3]) # easier viewing in TOPPView
        f.setPeptideIdentifications( [pepids[(i+k) % 3]] )
        fmap.push_back(f)
    pyopenms.FeatureXMLFile().store("output_%s.featureXML" % i, fmap)





from Types cimport *
from Feature cimport *
from DocumentIdentifier cimport *
from ProteinIdentification cimport *
from DataProcessing cimport *

cdef extern from "<OpenMS/KERNEL/FeatureMap.h>" namespace "OpenMS":
    
    cdef cppclass AnnotationStatistics "OpenMS::AnnotationStatistics":
        AnnotationStatistics() nogil except +
        AnnotationStatistics(AnnotationStatistics) nogil except +
        libcpp_vector[ size_t ] states
        bool operator==(AnnotationStatistics & rhs) nogil except +
        # NAMESPACE # AnnotationStatistics  operator+=(BaseFeature::AnnotationState state) nogil except +




from libcpp.vector cimport vector as libcpp_vector
from libcpp cimport bool
from libcpp.pair cimport pair as libcpp_pair

from Types cimport *
from DefaultParamHandler cimport *
from String cimport *

from ConsensusMap cimport *
from Feature cimport *
from FeatureMap cimport *
from MSExperiment cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *

from ProteinIdentification cimport *
from PeptideIdentification cimport *
from GaussFitter cimport *
from TextFile cimport *

cdef extern from "<OpenMS/MATH/STATISTICS/PosteriorErrorProbabilityModel.h>" namespace "OpenMS::Math":

    cdef cppclass PosteriorErrorProbabilityModel(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler

        PosteriorErrorProbabilityModel() nogil except +
        PosteriorErrorProbabilityModel(PosteriorErrorProbabilityModel) nogil except +   # wrap-ignore

        bool fit(libcpp_vector[double] & search_engine_scores, String outlier_handling) nogil except +
        bool fit(libcpp_vector[double] & search_engine_scores, libcpp_vector[double] & probabilities, String outlier_handling) nogil except +

        #Writes the distributions densities into the two vectors for a set of scores. Incorrect_densities represent the incorrectly assigned sequences.
        void fillDensities(libcpp_vector[double] & x_scores, libcpp_vector[double] & incorrect_density, libcpp_vector[double] & correct_density) nogil except +
        void fillLogDensities(libcpp_vector[double] & x_scores, libcpp_vector[double] & incorrect_density, libcpp_vector[double] & correct_density) nogil except +

        #computes the Maximum Likelihood with a log-likelihood function.
        double computeLogLikelihood(libcpp_vector[double] & incorrect_density, libcpp_vector[double] & correct_density) nogil except +

        libcpp_pair[ double, double ] pos_neg_mean_weighted_posteriors(libcpp_vector[double] &x_scores,
                                                                 libcpp_vector[double] &incorrect_posteriors);
        #returns estimated parameters for correctly assigned sequences. Fit should be used before.
        GaussFitResult getCorrectlyAssignedFitResult() nogil except +

        #returns estimated parameters for correctly assigned sequences. Fit should be used before.
        GaussFitResult getIncorrectlyAssignedFitResult() nogil except +

        #returns the estimated negative prior probability.
        double getNegativePrior() nogil except +

        #   Returns the computed posterior error probability for a given score.
        #   @note: fit has to be used before using this function. Otherwise this function will compute nonsense.
        double computeProbability(double score) nogil except +

        #initializes the plots
        TextFile initPlots(libcpp_vector[ double ] & x_scores) nogil except +

        # returns the gnuplot formula of the fitted gumbel distribution. Only
        # x0 and sigma are used as local parameter alpha and scale parameter
        # beta, respectively.
        String getGumbelGnuplotFormula(GaussFitResult & params) nogil except +

        # returns the gnuplot formula of the fitted gauss distribution.
        String getGaussGnuplotFormula(GaussFitResult & params) nogil except +

        # returns the gnuplot formula of the fitted mixture distribution.
        String getBothGnuplotFormula(GaussFitResult & incorrect, GaussFitResult & correct) nogil except +

        #plots the estimated distribution against target and decoy hits
        void plotTargetDecoyEstimation(libcpp_vector[double] & target, libcpp_vector[double] & decoy) nogil except +

        # returns the smallest score used in the last fit
        double getSmallestScore() nogil except +

        void tryGnuplot(const String & gp_file) nogil except +

        # Cannot handle bool by reference ... 
        # void updateScores(PosteriorErrorProbabilityModel PEP_model,
        #                   const String & search_engine,
        #                   const Int charge,
        #                   const bool prob_correct,
        #                   const bool split_charge,
        #                   libcpp_vector[ ProteinIdentification ] & protein_ids,
        #                   libcpp_vector[ PeptideIdentification ] & peptide_ids,
        #                   bool unable_to_fit_data,
        #                   bool data_might_not_be_well_fit) nogil except +




from libcpp cimport bool
from Types cimport *
from String cimport *
from IsotopeDistribution cimport *

cdef extern from "<OpenMS/CHEMISTRY/Element.h>" namespace "OpenMS":

    cdef cppclass Element:

        Element() nogil except +
        Element(Element) nogil except + # wrap-ignore

        # detailed constructor
        Element(String name,
                String symbol,
                UInt atomic_number,
                double average_weight,
                double mono_weight,
                IsotopeDistribution isotopes) nogil except +

        # sets unique atomic number
        void setAtomicNumber(UInt atomic_number) nogil except +

        # returns the unique atomic number
        UInt getAtomicNumber() nogil except +

        # sets the average weight of the element
        void setAverageWeight(double weight) nogil except +

        # returns the average weight of the element
        double getAverageWeight() nogil except +

        # sets the mono isotopic weight of the element
        void setMonoWeight(double weight) nogil except +

        # returns the mono isotopic weight of the element
        double getMonoWeight() nogil except +

        # sets the isotope distribution of the element
        void setIsotopeDistribution(IsotopeDistribution isotopes) nogil except +

        # returns the isotope distribution of the element
        IsotopeDistribution getIsotopeDistribution() nogil except +

        # set the name of the element
        void setName(String name) nogil except +

        # returns the name of the element
        String getName() nogil except +

        # sets symbol of the element
        void setSymbol(String symbol) nogil except +

        # returns symbol of the element
        String getSymbol() nogil except +




from String cimport *
from ExperimentalDesign cimport *
from libcpp cimport bool

cdef extern from "<OpenMS/FORMAT/ExperimentalDesignFile.h>" namespace "OpenMS":
    cdef cppclass ExperimentalDesignFile "OpenMS::ExperimentalDesignFile":
        ExperimentalDesignFile() nogil except + #wrap-ignore
        ExperimentalDesignFile(ExperimentalDesignFile&) nogil except + #wrap-ignore
                
# COMMENT: wrap static methods
cdef extern from "<OpenMS/FORMAT/ExperimentalDesignFile.h>" namespace "OpenMS::ExperimentalDesignFile":
    ExperimentalDesign load(const String& tsv_file, bool) nogil except + #wrap-attach:ExperimentalDesignFile




from LevMarqFitter1D cimport *
from InterpolationModel cimport *
from Peak1D cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/EmgFitter1D.h>" namespace "OpenMS":
    
    cdef cppclass EmgFitter1D(LevMarqFitter1D):
        # wrap-inherits:
        #  LevMarqFitter1D
        EmgFitter1D() nogil except +
        EmgFitter1D(EmgFitter1D) nogil except +
        # float fit1d(libcpp_vector[Peak1D] range_, InterpolationModel * & model) nogil except + # wrap-ignore
        # Fitter1D * create() nogil except +
        String getProductName() nogil except +




from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from libcpp.map cimport map as libcpp_map
from UniqueIdInterface cimport *
from ConsensusFeature cimport *
from FeatureMap cimport *
from MSExperiment cimport *
from Feature cimport *
from ProteinIdentification cimport *
from PeptideIdentification cimport *
from DataProcessing cimport *
from Types cimport *
from DocumentIdentifier cimport *
from RangeManager cimport *
from MetaInfoInterface cimport *

# this class has addons, see the ./addons folder

cdef extern from "<OpenMS/KERNEL/ConsensusMap.h>" namespace "OpenMS::ConsensusMap":

    cdef cppclass ColumnHeader(MetaInfoInterface):
        # wrap-inherits:
        #   MetaInfoInterface

        String filename
        String label
        Size size
        UInt64 unique_id

        ColumnHeader() nogil except +
        ColumnHeader(ColumnHeader &) nogil except +

    # for msvc++ compiler, see addons/ConsensusMap.pyx
    # ... forgot why Map[..] did not work
    ctypedef libcpp_map[unsigned long int, ColumnHeader] ColumnHeaders "OpenMS::ConsensusMap::ColumnHeaders"
    ctypedef libcpp_map[unsigned long int, ColumnHeader].iterator ColumnHeaders_iterator "OpenMS::ConsensusMap::ColumnHeaders::iterator"

cdef extern from "<OpenMS/KERNEL/ConsensusMap.h>" namespace "OpenMS":

    cdef cppclass ConsensusMap(UniqueIdInterface, DocumentIdentifier, RangeManager2, MetaInfoInterface):

        # wrap-inherits:
        #   UniqueIdInterface
        #   DocumentIdentifier
        #   RangeManager2
        #   MetaInfoInterface
        #
        # wrap-doc:
        #   A container for consensus elements.
        #   -----
        #   A ConsensusMap is a container holding 2-dimensional consensus elements
        #   (ConsensusFeature) which in turn represent analytes that have been
        #   quantified across multiple LC-MS/MS experiments. Each analyte in a
        #   ConsensusFeature is linked to its original LC-MS/MS run, the links are
        #   maintained by the ConsensusMap class.
        #   The map is implemented as a vector of elements of type ConsensusFeature.
        #   -----
        #   To be consistent, all maps who are referenced by ConsensusFeature objects
        #   (through a unique id) need to be registered in this class. 
        #   -----
        #   This class supports direct iteration in Python.

        ConsensusMap() nogil except +
        ConsensusMap(ConsensusMap &) nogil except +

        bool operator==(ConsensusMap) nogil except +
        bool operator!=(ConsensusMap) nogil except +

        int size() nogil except +
        bool empty() nogil except +
        void reserve(Size s) nogil except +
        ConsensusFeature & operator[](int) nogil except + #wrap-upper-limit:size()
        void push_back(ConsensusFeature spec) nogil except +

        ConsensusMap appendRows(ConsensusMap) nogil except +
        ConsensusMap appendColumns(ConsensusMap) nogil except +

        void clear(bool clear_meta_data) nogil except +
        void clear() nogil except +

        void updateRanges() nogil except +

        libcpp_vector[ProteinIdentification] getProteinIdentifications(
                ) nogil except+

        void setProteinIdentifications(
                libcpp_vector[ProteinIdentification]
                ) nogil except+

        libcpp_vector[PeptideIdentification]\
                getUnassignedPeptideIdentifications() nogil except+

        void setUnassignedPeptideIdentifications(
                libcpp_vector[PeptideIdentification]
                ) nogil except+

        libcpp_vector[DataProcessing] getDataProcessing() nogil except +
        void setDataProcessing(libcpp_vector[DataProcessing])   nogil except +

        void setPrimaryMSRunPath(StringList& s) nogil except +
        void setPrimaryMSRunPath(StringList& s, MSExperiment& e) nogil except +
        void getPrimaryMSRunPath(StringList& toFill) nogil except +

        libcpp_vector[ConsensusFeature].iterator begin(
                ) nogil except +   # wrap-iter-begin:__iter__(ConsensusFeature)
        libcpp_vector[ConsensusFeature].iterator end(
                )   nogil except +  # wrap-iter-end:__iter__(ConsensusFeature)

        # wrapped in ../addons/ConsensusMap.pyx:
        void applyMemberFunction(Size(* fun)()) nogil except + # wrap-ignore

        void sortByIntensity(bool reverse) nogil except +
        void sortByIntensity() nogil except +
        void sortByRT() nogil except +
        void sortByMZ() nogil except +
        void sortByPosition() nogil except +
        void sortByQuality(bool reverse) nogil except +
        void sortByQuality() nogil except +
        void sortBySize() nogil except +
        void sortByMaps() nogil except +

        # wrapped in ../addons/ConsensusMap.pyx:
        void setColumnHeaders(ColumnHeaders &)   #wrap-ignore
        ColumnHeaders & getColumnHeaders()       #wrap-ignore

        String getExperimentType() nogil except +
        void setExperimentType(String experiment_type) nogil except +

        void sortPeptideIdentificationsByMapIndex() nogil except +




from MSSpectrum cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from Param cimport *
from DefaultParamHandler cimport *
from ProgressLogger cimport *

cdef extern from "<OpenMS/FILTERING/CALIBRATION/TOFCalibration.h>" namespace "OpenMS":

    cdef cppclass TOFCalibration(DefaultParamHandler, ProgressLogger):
        # wrap-inherits:
        #    DefaultParamHandler
        #    ProgressLogger

        TOFCalibration()                  nogil except +
        TOFCalibration(TOFCalibration)   nogil except + #wrap-ignore

        void calibrate(MSExperiment & input, MSExperiment & output, libcpp_vector[double] & exp_masses) nogil except +
        void pickAndCalibrate(MSExperiment & input, MSExperiment & output, libcpp_vector[double] & exp_masses) nogil except +

        libcpp_vector[ double ]  getML1s() nogil except +
        void setML1s(libcpp_vector[ double ] & ml1s) nogil except +
        libcpp_vector[ double ]  getML2s() nogil except +
        void setML2s(libcpp_vector[ double ] & ml2s) nogil except +
        libcpp_vector[ double ]  getML3s() nogil except +
        void setML3s(libcpp_vector[ double ] & ml3s) nogil except +




from smart_ptr cimport shared_ptr
from Types cimport *
from OpenSwathDataStructures cimport *
from SpectrumAccessOpenMS cimport *
from LightTargetedExperiment cimport LightTransition

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/OpenSwathScoring.h>" namespace "OpenMS":

    cdef cppclass OpenSwathScoring:

        OpenSwathScoring() nogil except +
        OpenSwathScoring(OpenSwathScoring) nogil except + # wrap-ignore

        void initialize(double rt_normalization_factor,
                        int add_up_spectra,
                        double spacing_for_spectra_resampling,
                        double drift_extra,
                        OpenSwath_Scores_Usage su,
                        libcpp_string spectrum_addition_method) nogil except +

        # void calculateChromatographicScores(
        #       OpenSwath::IMRMFeature* imrmfeature,
        #       const std::vector<std::string>& native_ids,
        #       const std::vector<double>& normalized_library_intensity,
        #       std::vector<OpenSwath::ISignalToNoisePtr>& signal_noise_estimators,
        #       OpenSwath_Scores & scores) nogil except +

        # void calculateLibraryScores(
        #       OpenSwath::IMRMFeature* imrmfeature,
        #       const std::vector<TransitionType> & transitions,
        #       const PeptideType& pep,
        #       const double normalized_feature_rt,
        #       OpenSwath_Scores & scores) nogil except +

        # void calculateDIAScores(OpenSwath::IMRMFeature* imrmfeature, 
        #     const std::vector<TransitionType> & transitions,
        #     OpenSwath::SpectrumAccessPtr swath_map,
        #     OpenSwath::SpectrumAccessPtr ms1_map,
        #     OpenMS::DIAScoring & diascoring,
        #     const PeptideType& pep,
        #     OpenSwath_Scores & scores) nogil except +

        void getNormalized_library_intensities_(libcpp_vector[LightTransition] transitions,
                                                libcpp_vector[double] normalized_library_intensity) nogil except +

    cdef cppclass OpenSwath_Scores_Usage:

        OpenSwath_Scores_Usage() nogil except +
        OpenSwath_Scores_Usage(OpenSwath_Scores_Usage) nogil except + # wrap-ignore

        bool use_coelution_score_
        bool use_shape_score_
        bool use_rt_score_
        bool use_library_score_
        bool use_elution_model_score_
        bool use_intensity_score_
        bool use_total_xic_score_
        bool use_total_mi_score_
        bool use_nr_peaks_score_
        bool use_sn_score_
        bool use_mi_score_
        bool use_dia_scores_
        bool use_sonar_scores
        bool use_ms1_correlation
        bool use_ms1_fullscan
        bool use_ms1_mi
        bool use_uis_scores

    cdef cppclass OpenSwath_Scores:

        OpenSwath_Scores() nogil except +
        OpenSwath_Scores(OpenSwath_Scores) nogil except + # wrap-ignore

        double get_quick_lda_score(double library_corr_, double
                                   library_norm_manhattan_, double
                                   norm_rt_score_, double
                                   xcorr_coelution_score_, double
                                   xcorr_shape_score_, double log_sn_score_) nogil except +

        double calculate_lda_prescore(OpenSwath_Scores scores) nogil except +
        double calculate_swath_lda_prescore(OpenSwath_Scores scores) nogil except +

        double elution_model_fit_score
        double library_corr
        double library_norm_manhattan
        double library_rootmeansquare
        double library_sangle
        double norm_rt_score

        double isotope_correlation
        double isotope_overlap
        double massdev_score
        double xcorr_coelution_score
        double xcorr_shape_score

        double yseries_score
        double bseries_score
        double log_sn_score

        double weighted_coelution_score
        double weighted_xcorr_shape
        double weighted_massdev_score
       
        double ms1_xcorr_coelution_score
        double ms1_xcorr_coelution_contrast_score
        double ms1_xcorr_coelution_combined_score
        double ms1_xcorr_shape_score
        double ms1_xcorr_shape_contrast_score
        double ms1_xcorr_shape_combined_score
        double ms1_ppm_score
        double ms1_isotope_correlation
        double ms1_isotope_overlap
        double ms1_mi_score
        double ms1_mi_contrast_score
        double ms1_mi_combined_score

        double sonar_sn 
        double sonar_diff
        double sonar_trend
        double sonar_rsq
        double sonar_shape
        double sonar_lag

        double library_manhattan
        double library_dotprod
        double intensity
        double total_xic
        double nr_peaks
        double sn_ratio
        double mi_score 
        double weighted_mi_score

        double rt_difference
        double normalized_experimental_rt
        double raw_rt_score

        double dotprod_score_dia
        double manhatt_score_dia





    def transform(self, path, transformer):
        assert (isinstance(path, str) or isinstance(path, unicode) or isinstance(path, bytes) or isinstance(path, String)), 'arg path wrong type'

        #
        # the referenced functions _wrap_MSSpectrum, _wrap_MSChromatogram and
        # _wrap_ExperimentalSettings are declared in the MzMLFile.pyx file!
        #

        assert hasattr(transformer, "consumeSpectrum")
        assert hasattr(transformer, "consumeChromatogram")
        assert hasattr(transformer, "setExpectedSize")
        assert hasattr(transformer, "setExperimentalSettings")
        cdef _PythonMSDataConsumer * consumer
        consumer = new _PythonMSDataConsumer(transformer,
                                             _wrap_MSSpectrum_mzxml,
                                             _wrap_MSChromatogram_mzxml,
                                             _wrap_ExperimentalSettings_mzxml)

        try:
            self.inst.get().transform(deref((convString(path)).get()), consumer)
        finally:
            del consumer

cdef _wrap_MSSpectrum_mzxml(const _MSSpectrum & _spec):
    cdef MSSpectrum spec = MSSpectrum.__new__(MSSpectrum)
    spec.inst = shared_ptr[_MSSpectrum](new _MSSpectrum(_spec))
    return spec


cdef _wrap_MSChromatogram_mzxml(const _MSChromatogram & _chromo):
    cdef MSChromatogram chromo = MSChromatogram.__new__(MSChromatogram)
    chromo.inst = shared_ptr[_MSChromatogram](new _MSChromatogram(_chromo))
    return chromo


cdef _wrap_ExperimentalSettings_mzxml(const _ExperimentalSettings & _exp):
    cdef ExperimentalSettings exp = ExperimentalSettings.__new__(ExperimentalSettings)
    exp.inst = shared_ptr[_ExperimentalSettings](new _ExperimentalSettings(_exp))
    return exp



from String cimport *
from MRMFeatureQC cimport *

cdef extern from "<OpenMS/FORMAT/MRMFeatureQCFile.h>" namespace "OpenMS":

    cdef cppclass MRMFeatureQCFile:

        MRMFeatureQCFile() nogil except +
        MRMFeatureQCFile(MRMFeatureQCFile &) nogil except +

        void load(const String& filename, MRMFeatureQC& mrmfqc, const bool is_component_group) nogil except +
        # void store(String filename, MRMFeatureQC mrmfqc) nogil except +



import unittest
import os

import pyopenms

class TestPepXML(unittest.TestCase):

    def setUp(self):
        dirname = os.path.dirname(os.path.abspath(__file__))
        self.filename = os.path.join(dirname, "test.pep.xml").encode()

    def test_readfile(self):

        pepxml_file = pyopenms.PepXMLFile()
        peps = []
        prots = []
        pepxml_file.load(self.filename, prots, peps)

    def test_readfile_content(self):

        pepxml_file = pyopenms.PepXMLFile()
        peps = []
        prots = []
        pepxml_file.load(self.filename, prots, peps)

        self.assertEqual( len(prots),  3)
        self.assertEqual( len(peps),  19)

        self.assertEqual( peps[0].getHits()[0].getSequence().toString(), ".(Glu->pyro-Glu)ELNKEMAAEKAKAAAG")

class TestIdXML(unittest.TestCase):

    def setUp(self):
        dirname = os.path.dirname(os.path.abspath(__file__))
        self.filename = os.path.join(dirname, "test.idXML").encode()

    def test_readfile(self):
        idxml_file = pyopenms.IdXMLFile()
        peps = []
        prots = []
        idxml_file.load(self.filename, prots, peps)

    def test_readfile_content(self):
        idxml_file = pyopenms.IdXMLFile()
        peps = []
        prots = []
        idxml_file.load(self.filename, prots, peps)

        self.assertEqual( len(prots),  1)
        self.assertEqual( len(peps),  3)

class TestIndexedMzMLFileLoader(unittest.TestCase):

    def setUp(self):
        dirname = os.path.dirname(os.path.abspath(__file__))
        self.filename = os.path.join(dirname, "test.indexed.mzML").encode()

    def test_readfile(self):
        e = pyopenms.OnDiscMSExperiment();
        success = pyopenms.IndexedMzMLFileLoader().load(self.filename, e)

        self.assertTrue(success)

    def test_readfile_content(self):
        e = pyopenms.OnDiscMSExperiment();
        pyopenms.IndexedMzMLFileLoader().load(self.filename, e)

        self.assertEqual( e.getNrSpectra() ,  2)
        self.assertEqual( e.getNrChromatograms() ,  1)

        s = e.getSpectrum(0)
        data_mz, data_int = s.get_peaks()
        self.assertEqual( len(data_mz), 19914)
        self.assertEqual( len(data_int), 19914)

        self.assertEqual( len(e.getSpectrum(1).get_peaks()[0]), 19800)
        self.assertEqual( len(e.getSpectrum(1).get_peaks()[1]), 19800)
        self.assertEqual( len(e.getChromatogram(0).get_peaks()[0]), 48)
        self.assertEqual( len(e.getChromatogram(0).get_peaks()[1]), 48)

        if False:
            # Currently we don't deal with exceptions properly
            raised = False
            try:
                e.getChromatogram(2).get_peaks()
            except Exception as e:
                raised = True

            self.assertTrue(raised)

class TestIndexedMzMLFile(unittest.TestCase):

    def setUp(self):
        dirname = os.path.dirname(os.path.abspath(__file__))
        self.filename = os.path.join(dirname, "test.indexed.mzML").encode()

    def test_readfile(self):
        f = pyopenms.IndexedMzMLHandler()
        f.openFile(self.filename)

        self.assertTrue(f.getParsingSuccess())

    def test_readfile_content(self):
        f = pyopenms.IndexedMzMLHandler()
        f.openFile(self.filename)

        self.assertEqual( f.getNrSpectra() ,  2)
        self.assertEqual( f.getNrChromatograms() ,  1)

        s = f.getSpectrumById(0)
        mzdata = s.getMZArray()
        intdata = s.getIntensityArray()
        self.assertEqual( len(mzdata), 19914)
        self.assertEqual( len(intdata), 19914)

        s = f.getMSSpectrumById(0)
        self.assertEqual(s.size(), 19914)
        s = f.getMSSpectrumById(1)
        self.assertEqual(s.size(), 19800)
        c = f.getMSChromatogramById(0)
        self.assertEqual(c.size(), 48)

if __name__ == '__main__':
    unittest.main()




from Types cimport *
from ConsensusIDAlgorithmIdentity cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/ConsensusIDAlgorithmBest.h>" namespace "OpenMS":
    
    cdef cppclass ConsensusIDAlgorithmBest(ConsensusIDAlgorithmIdentity) :
        # wrap-inherits:
        #  ConsensusIDAlgorithmIdentity
        ConsensusIDAlgorithmBest() nogil except +
        ConsensusIDAlgorithmBest(ConsensusIDAlgorithmBest) nogil except + #wrap-ignore




from Types cimport *
from libcpp cimport bool
from PeptideHit cimport *
from MSExperiment cimport *
from ResidueModification cimport *
from FASTAFile cimport *
from ProteaseDigestion cimport *

# cdef extern from "<OpenMS/ANALYSIS/XLMS/OPXLDataStructs.h>" namespace "OpenMS::OPXLDataStructs":
#     
#     cdef cppclass XLPrecursorComparator "OpenMS::OPXLDataStructs::XLPrecursorComparator":
#         XLPrecursorComparator(XLPrecursorComparator) nogil except + #wrap-ignore
#         bool operator()(XLPrecursor & a, XLPrecursor & b) nogil except +
#         bool operator()(XLPrecursor & a, double & b) nogil except +
#         bool operator()(double & a, XLPrecursor & b) nogil except +
# 



from Types cimport *
from ConsensusIDAlgorithmIdentity cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/ConsensusIDAlgorithmRanks.h>" namespace "OpenMS":
    
    cdef cppclass ConsensusIDAlgorithmRanks(ConsensusIDAlgorithmIdentity) :
        # wrap-inherits:
        #  ConsensusIDAlgorithmIdentity
        ConsensusIDAlgorithmRanks() nogil except +
        ConsensusIDAlgorithmRanks(ConsensusIDAlgorithmRanks) nogil except + #wrap-ignore






    def create(self):
        cdef IsotopeMarker res = IsotopeMarker.__new__(IsotopeMarker)
        res.inst = shared_ptr[_IsotopeMarker]( <_IsotopeMarker *> self.inst.get().create() )
        return res



from DefaultParamHandler cimport *
from ConsensusMap cimport *
from FeatureMap cimport *
from Feature cimport *
from libcpp.vector cimport vector as libcpp_vector

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/FeatureGroupingAlgorithm.h>" namespace "OpenMS":

    cdef cppclass FeatureGroupingAlgorithm(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler
        # wrap-ignore
        # Abstract Class
        void transferSubelements(libcpp_vector[ConsensusMap] maps,
                                 ConsensusMap & out
                                ) nogil except +

        # since this is a base class, cannot have overloaded methods
        # void group(libcpp_vector[ FeatureMap ] & maps, ConsensusMap & out)
        # void group(libcpp_vector[ ConsensusMap ] & maps, ConsensusMap & out)
        void registerChildren() nogil except +



from Types cimport *
from libcpp cimport bool
from libcpp.set cimport set as libcpp_set
from EmpiricalFormula cimport *
from String cimport *

cdef extern from "<OpenMS/CHEMISTRY/DigestionEnzyme.h>" namespace "OpenMS":
    
    cdef cppclass DigestionEnzyme "OpenMS::DigestionEnzyme":
        DigestionEnzyme(DigestionEnzyme) nogil except +
        DigestionEnzyme(const String & name, const String & cleavage_regex, libcpp_set[ String ] & synonyms, String regex_description) nogil except +
        void setName(const String & name) nogil except +
        String getName() nogil except +
        void setSynonyms(libcpp_set[ String ] & synonyms) nogil except +
        void addSynonym(const String & synonym) nogil except +
        libcpp_set[ String ] getSynonyms() nogil except +
        void setRegEx(const String & cleavage_regex) nogil except +
        String getRegEx() nogil except +
        void setRegExDescription(const String & value) nogil except +
        String getRegExDescription() nogil except +
        bool operator==(const DigestionEnzyme & enzyme) nogil except +
        bool operator!=(const DigestionEnzyme & enzyme) nogil except +
        bool operator==(const String & cleavage_regex) nogil except +
        bool operator!=(const String & cleavage_regex) nogil except +
        bool operator<(const DigestionEnzyme & enzyme) nogil except +
        bool setValueFromFile(const String & key, const String & value) nogil except +




from Types cimport *
from String cimport *

cdef extern from "<OpenMS/METADATA/MetaInfoRegistry.h>" namespace "OpenMS":
    
    cdef cppclass MetaInfoRegistry "OpenMS::MetaInfoRegistry":
        MetaInfoRegistry() nogil except +
        MetaInfoRegistry(MetaInfoRegistry) nogil except +
        UInt registerName(const String & name, const String & description, const String & unit) nogil except +
        void setDescription(UInt index, const String & description) nogil except +
        void setDescription(const String & name, const String & description) nogil except +
        void setUnit(UInt index, const String & unit) nogil except +
        void setUnit(const String & name, const String & unit) nogil except +
        UInt getIndex(const String & name) nogil except +
        String getName(UInt index) nogil except +
        String getDescription(UInt index) nogil except +
        String getDescription(const String & name) nogil except +
        String getUnit(UInt index) nogil except +
        String getUnit(const String & name) nogil except +




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from String cimport *
from SiriusAdapterHit cimport *
from MzTab cimport *

cdef extern from "<OpenMS/FORMAT/DATAACCESS/SiriusMzTabWriter.h>" namespace "OpenMS":
    
    cdef cppclass SiriusMzTabWriter "OpenMS::SiriusMzTabWriter":
        SiriusMzTabWriter() nogil except + 
        SiriusMzTabWriter(SiriusMzTabWriter) nogil except + #wrap-ignore
        String extract_scan_index(const String & path) nogil except +
        void read(libcpp_vector[String] sirius_output_paths, String original_input_mzml, Size top_n_hits, MzTab & result) nogil except +


cdef extern from "<OpenMS/FORMAT/DATAACCESS/SiriusMzTabWriter.h>" namespace "OpenMS::SiriusMzTabWriter":
    
    cdef cppclass SiriusAdapterRun "OpenMS::SiriusMzTabWriter::SiriusAdapterRun":
        SiriusAdapterRun() nogil except + 
        SiriusAdapterRun(SiriusAdapterRun) nogil except + #wrap-ignore
        libcpp_vector[ SiriusAdapterIdentification ] identifications


cdef extern from "<OpenMS/FORMAT/DATAACCESS/SiriusMzTabWriter.h>" namespace "OpenMS::SiriusMzTabWriter":
    
    cdef cppclass SiriusAdapterIdentification "OpenMS::SiriusMzTabWriter::SiriusAdapterIdentification":
        SiriusAdapterIdentification() nogil except +
        SiriusAdapterIdentification(SiriusAdapterIdentification) nogil except + # wrap-ignore

        int scan_index
        int scan_number
        String feature_id
        libcpp_vector[ SiriusAdapterHit ] hits




import unittest
import os

import pyopenms

class TestTraMLFile(unittest.TestCase):

    def setUp(self):
        dirname = os.path.dirname(os.path.abspath(__file__))
        self.filename = os.path.join(dirname, "test.TraML").encode()

    def test_readfile(self):
        targeted = pyopenms.TargetedExperiment();
        tramlfile = pyopenms.TraMLFile();
        tramlfile.load(self.filename, targeted);

    def test_readfile_content(self):
        targeted = pyopenms.TargetedExperiment();
        tramlfile = pyopenms.TraMLFile();
        tramlfile.load(self.filename, targeted);
        self.assertEqual(len( targeted.getTransitions() ), 3 )

        self.assertAlmostEqual(targeted.getTransitions()[0].getPrecursorMZ(), 500.0)
        self.assertAlmostEqual(targeted.getTransitions()[0].getProductMZ(), 628.45, places=4)
        self.assertEqual(targeted.getTransitions()[0].getName(), "tr1" )
        self.assertEqual(targeted.getTransitions()[0].getNativeID(), "tr1" )
        self.assertEqual(targeted.getTransitions()[0].getPeptideRef(), "tr_gr1")

    def test_TargetedExperiment(self):
        targeted = pyopenms.TargetedExperiment();
        tramlfile = pyopenms.TraMLFile();
        tramlfile.load(self.filename, targeted);
        self.assertEqual(len( targeted.getTransitions() ), 3 )

        targeted.setCVs(targeted.getCVs())
        targeted.setTargetCVTerms(targeted.getTargetCVTerms())
        targeted.setPeptides(targeted.getPeptides())
        targeted.setProteins(targeted.getProteins())
        targeted.setTransitions(targeted.getTransitions())

        first_transition = targeted.getTransitions()[0] 
        first_peptide = targeted.getPeptides()[0] 
        targeted.addTransition(first_transition)
        targeted.addPeptide(first_peptide)

        self.assertTrue( targeted.getPeptideByRef(first_transition.getPeptideRef()) is not None)
        self.assertTrue( targeted.getProteinByRef(first_peptide.protein_refs[0]) is not None)


if __name__ == '__main__':
    unittest.main()



from Types cimport *
from libcpp cimport bool
from String cimport *
from MetaInfoInterface cimport *

cdef extern from "<OpenMS/METADATA/SampleTreatment.h>" namespace "OpenMS":
    
    cdef cppclass SampleTreatment(MetaInfoInterface) :
        # wrap-ignore
        # no-pxd-import
        # ABSTRACT class
        #
        # wrap-inherits:
        #  MetaInfoInterface

        SampleTreatment(SampleTreatment) nogil except +
        SampleTreatment(const String & type_) nogil except +

        bool operator==(SampleTreatment & rhs) nogil except +
        String getType() nogil except +
        String getComment() nogil except +
        void setComment(const String & comment) nogil except +
        # POINTER # SampleTreatment * clone() nogil except +





from libcpp.vector cimport vector as libcpp_vector
from FeatureMap cimport *
from MZTrafoModel cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from PeptideIdentification cimport *
from ProgressLogger cimport *
from Types cimport *
from Precursor cimport *

cdef extern from "<OpenMS/FILTERING/CALIBRATION/InternalCalibration.h>" namespace "OpenMS":

    cdef cppclass InternalCalibration(ProgressLogger):
        # wrap-inherits:
        #    ProgressLogger

        InternalCalibration()      nogil except +
        InternalCalibration(InternalCalibration) nogil except +


        Size fillCalibrants(MSExperiment,
                            libcpp_vector[InternalCalibration_LockMass],
                            double tol_ppm,
                            bool lock_require_mono,
                            bool lock_require_iso,
                            CalibrationData& failed_lock_masses,
                            bool verbose) nogil except +
        Size fillCalibrants(FeatureMap, double) nogil except +
        Size fillCalibrants(libcpp_vector[PeptideIdentification], double) nogil except +
        CalibrationData getCalibrationPoints() nogil except +
        bool calibrate(MSExperiment,
                       libcpp_vector[int],
                       MZTrafoModel_MODELTYPE,
                       double rt_chunk,
                       bool use_RANSAC,
                       double post_ppm_median,
                       double post_ppm_MAD,
                       String file_models,
                       String file_models_plot,
                       String file_residuals,
                       String file_residuals_plot,
                       String rscript_executable) nogil except +



## wrap static methods
cdef extern from "<OpenMS/FILTERING/CALIBRATION/InternalCalibration.h>" namespace "OpenMS::InternalCalibration":

    void applyTransformation(libcpp_vector[Precursor]& pcs,
                             MZTrafoModel& trafo) nogil except + # wrap-attach:InternalCalibration
    void applyTransformation(MSSpectrum & spec, IntList& target_mslvl,
                             MZTrafoModel & trafo) nogil except + # wrap-attach:InternalCalibration
    void applyTransformation(MSExperiment & exp,
                             IntList& target_mslvl, MZTrafoModel& trafo) nogil except + # wrap-attach:InternalCalibration

cdef extern from "<OpenMS/FILTERING/CALIBRATION/InternalCalibration.h>" namespace "OpenMS::InternalCalibration":

    cdef cppclass InternalCalibration_LockMass "OpenMS::InternalCalibration::LockMass":
        #InternalCalibration_LockMass() nogil except +
        InternalCalibration_LockMass(InternalCalibration_LockMass) nogil except + # wrap-ignore
        InternalCalibration_LockMass(double mz_, int lvl_, int charge_) nogil except +
        double mz # m/z of the lock mass (incl. adducts)
        unsigned int ms_level # MS level where it occurs
        int charge # charge of the ion (to find isotopes)




from PeptideIdentification cimport *
from DefaultParamHandler cimport *
from Map cimport *
# from ZhangSimilarityScore cimport *
# from MassDecomposition cimport *
# from MassDecompositionAlgorithm cimport *
# from CompNovoIonScoringBase cimport *

cdef extern from "<OpenMS/ANALYSIS/DENOVO/CompNovoIonScoring.h>" namespace "OpenMS":
    
    cdef cppclass CompNovoIonScoring "OpenMS::CompNovoIonScoring":
        CompNovoIonScoring() nogil except +
        CompNovoIonScoring(CompNovoIonScoring) nogil except +
        # TODO -> replace type ... 
        # TODO OpenMS Map type
        # void scoreSpectra(Map[ double, IonScore ] &CID_ion_scores, MSSpectrum & CID_spec,
        #   MSSpectrum &ETD_spec, double precursor_weight, Size charge) nogil except +




from MSSpectrum cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from Param cimport *
from DefaultParamHandler cimport *
from ProgressLogger cimport *

cdef extern from "<OpenMS/FILTERING/TRANSFORMERS/LinearResampler.h>" namespace "OpenMS":

    cdef cppclass LinearResampler(DefaultParamHandler, ProgressLogger):
        # wrap-inherits:
        #    DefaultParamHandler
        #    ProgressLogger

        LinearResampler()                  nogil except +
        LinearResampler(LinearResampler)   nogil except + #wrap-ignore
        void raster(MSSpectrum & input) nogil except +
        void rasterExperiment(MSExperiment & input) nogil except +




import unittest
import math
import random

import pyopenms


class TestBilinearInterpolation(unittest.TestCase):

    def test_BilinearInterpolation(self):
        mat = pyopenms.MatrixDouble()
        mat.resize(2, 3, float())
        mat.setValue(0, 0, 17)
        mat.setValue(0, 1, 18.9)
        mat.setValue(0, 2, 20.333)
        mat.setValue(1, 0, -0.1)
        mat.setValue(1, 1, -0.13)
        mat.setValue(1, 2, -0.001)

        bilip = pyopenms.BilinearInterpolation()
        bilip.setData(mat)
        bilip.setMapping_0(13.0, 230.0, 14.0, 250.0)
        bilip.setMapping_1(15.0, 2100.0, 17.0, 2900.0)

        bilip_2 = pyopenms.BilinearInterpolation()
        bilip_2 = bilip

        self.assertAlmostEqual(bilip_2.getScale_0(), 20)
        self.assertAlmostEqual(bilip_2.getScale_1(), 400)
        self.assertAlmostEqual(bilip_2.getOffset_0(), -30)
        self.assertAlmostEqual(bilip_2.getOffset_1(), -3900)
        self.assertAlmostEqual(bilip_2.getInsideReferencePoint_0(), 13)
        self.assertAlmostEqual(bilip_2.getOutsideReferencePoint_0(), 230)
        self.assertAlmostEqual(bilip_2.getInsideReferencePoint_1(), 15)
        self.assertAlmostEqual(bilip_2.getOutsideReferencePoint_1(), 2100)

        for i in range(mat.rows()):
            for j in range(mat.cols()):
                self.assertAlmostEqual(bilip.getData().getValue(i, j),
                                       mat.getValue(i, j))

    # same test for getData and setData
    def test_getData_setData(self):
        bilip = pyopenms.BilinearInterpolation()

        tmp = bilip.getData()
        tmp.resize(2, 3, float())
        tmp.setValue(1, 2, 10012)
        tmp.setValue(0, 0, 10000)
        tmp.setValue(1, 0, 10010)
        bilip.setData(tmp)

        bilip_2 = pyopenms.BilinearInterpolation(bilip)
        self.assertAlmostEqual(bilip_2.getData().getValue(1, 2), 10012)
        self.assertAlmostEqual(bilip_2.getData().getValue(0, 0), 10000)
        self.assertAlmostEqual(bilip_2.getData().getValue(1, 0), 10010)

    def test_setMapping_0(self):
        bilip = pyopenms.BilinearInterpolation()
        bilip.setMapping_0(1.0, 2.0, 3.0, 8.0)

        self.assertAlmostEqual(bilip.getScale_0(), 3)
        self.assertAlmostEqual(bilip.getOffset_0(), -1)
        self.assertAlmostEqual(bilip.getScale_1(), 1)
        self.assertAlmostEqual(bilip.getOffset_1(), 0)

        bilip2 = pyopenms.BilinearInterpolation()
        bilip2.setMapping_0(3.0, 1.0, 2.0)

        self.assertAlmostEqual(bilip2.getScale_0(), 3)
        self.assertAlmostEqual(bilip2.getOffset_0(), -1)
        self.assertAlmostEqual(bilip2.getScale_1(), 1)
        self.assertAlmostEqual(bilip2.getOffset_1(), 0)

    def test_setOffset_0(self):
        bilip = pyopenms.BilinearInterpolation()
        bilip.setOffset_0(987)
        self.assertAlmostEqual(bilip.getOffset_0(), 987)

    def test_setScale_0(self):
        bilip = pyopenms.BilinearInterpolation()
        bilip.setScale_0(987)
        self.assertAlmostEqual(bilip.getScale_0(), 987)

    def test_getInsideReferencePoint_0(self):
        bilip = pyopenms.BilinearInterpolation()
        bilip.setMapping_0(1.0, 4.0, 3.0, 8.0)

        self.assertAlmostEqual(bilip.getInsideReferencePoint_0(), 1)
        self.assertAlmostEqual(bilip.getOutsideReferencePoint_0(), 4)
        self.assertAlmostEqual(bilip.getInsideReferencePoint_1(), 0)
        self.assertAlmostEqual(bilip.getOutsideReferencePoint_1(), 0)

    def test_index2key_0(self):
        bilip = pyopenms.BilinearInterpolation()
        bilip.setMapping_0(3.0, 1.0, 2.0)
        self.assertAlmostEqual(bilip.index2key_0(0), -1)
        self.assertAlmostEqual(bilip.index2key_1(0), 0)

    def test_key2index_0(self):
        bilip = pyopenms.BilinearInterpolation()
        bilip.setMapping_0(3.0, 1.0, 2.0)
        self.assertAlmostEqual(bilip.key2index_0(-1), 0)
        self.assertAlmostEqual(bilip.key2index_1(0), 0)

    def test_supportMax_0(self):
        bilip = pyopenms.BilinearInterpolation()
        bilip.setMapping_0(3.0, 1.0, 2.0)
        bilip.setMapping_1(5.0, 3.0, 4.0)

        tmp = bilip.getData()
        tmp.resize(2, 3, float())
        bilip.setData(tmp)
        self.assertAlmostEqual(bilip.index2key_0(0), -1)
        self.assertAlmostEqual(bilip.index2key_0(1), 2)
        self.assertAlmostEqual(bilip.supportMin_0(), -4)
        self.assertAlmostEqual(bilip.supportMax_0(), 5)

        self.assertAlmostEqual(bilip.index2key_1(0), -11)
        self.assertAlmostEqual(bilip.index2key_1(2), -1)
        self.assertAlmostEqual(bilip.supportMin_1(), -16)
        self.assertAlmostEqual(bilip.supportMax_1(), 4)

    def test_setMapping_1(self):
        bilip = pyopenms.BilinearInterpolation()
        bilip.setMapping_1(1.0, 2.0, 3.0, 8.0)

        self.assertAlmostEqual(bilip.getScale_1(), 3)
        self.assertAlmostEqual(bilip.getOffset_1(), -1)
        self.assertAlmostEqual(bilip.getScale_0(), 1)
        self.assertAlmostEqual(bilip.getOffset_0(), 0)

        bilip2 = pyopenms.BilinearInterpolation()
        bilip2.setMapping_1(3.0, 1.0, 2.0)

        self.assertAlmostEqual(bilip2.getScale_1(), 3)
        self.assertAlmostEqual(bilip2.getOffset_1(), -1)
        self.assertAlmostEqual(bilip2.getScale_0(), 1)
        self.assertAlmostEqual(bilip2.getOffset_0(), 0)

    def test_setOffset_1(self):
        bilip = pyopenms.BilinearInterpolation()
        bilip.setOffset_1(987)
        self.assertAlmostEqual(bilip.getOffset_1(), 987)

    def test_setScale_1(self):
        bilip = pyopenms.BilinearInterpolation()
        bilip.setScale_1(987)
        self.assertAlmostEqual(bilip.getScale_1(), 987)

    def test_getInsideReferencePoint_1(self):
        bilip = pyopenms.BilinearInterpolation()
        bilip.setMapping_1(1.0, 4.0, 3.0, 8.0)

        self.assertAlmostEqual(bilip.getInsideReferencePoint_1(), 1)
        self.assertAlmostEqual(bilip.getOutsideReferencePoint_1(), 4)
        self.assertAlmostEqual(bilip.getInsideReferencePoint_0(), 0)
        self.assertAlmostEqual(bilip.getOutsideReferencePoint_0(), 0)

    def test_index2key_1(self):
        bilip = pyopenms.BilinearInterpolation()
        bilip.setMapping_1(3.0, 1.0, 2.0)
        self.assertAlmostEqual(bilip.index2key_1(0), -1)
        self.assertAlmostEqual(bilip.index2key_0(0), 0)

    def test_key2index_1(self):
        bilip = pyopenms.BilinearInterpolation()
        bilip.setMapping_1(3.0, 1.0, 2.0)
        self.assertAlmostEqual(bilip.key2index_1(-1), 0)
        self.assertAlmostEqual(bilip.key2index_0(0), 0)

    def test_supportMax_1(self):
        bilip = pyopenms.BilinearInterpolation()
        bilip.setMapping_1(3.0, 1.0, 2.0)
        bilip.setMapping_0(5.0, 3.0, 4.0)

        tmp = bilip.getData()
        tmp.resize(3, 2, float())
        bilip.setData(tmp)
        self.assertAlmostEqual(bilip.index2key_1(0), -1)
        self.assertAlmostEqual(bilip.index2key_1(1), 2)
        self.assertAlmostEqual(bilip.supportMin_1(), -4)
        self.assertAlmostEqual(bilip.supportMax_1(), 5)

        self.assertAlmostEqual(bilip.index2key_0(0), -11)
        self.assertAlmostEqual(bilip.index2key_0(2), -1)
        self.assertAlmostEqual(bilip.supportMin_0(), -16)
        self.assertAlmostEqual(bilip.supportMax_0(), 4)

    def test_empty(self):
        bilip = pyopenms.BilinearInterpolation()
        self.assertTrue(bilip.empty())

        tmp = bilip.getData()
        tmp.resize(1, 2, float())
        bilip.setData(tmp)
        self.assertFalse(bilip.empty())

        tmp = bilip.getData()
        tmp.resize(0, 0, float())
        bilip.setData(tmp)
        self.assertTrue(bilip.empty())

        tmp = bilip.getData()
        tmp.resize(1, 2, float())
        bilip.setData(tmp)
        self.assertFalse(bilip.empty())

        tmp = bilip.getData()
        tmp.resize(1, 0, float())
        bilip.setData(tmp)
        self.assertTrue(bilip.empty())

        tmp = bilip.getData()
        tmp.resize(1, 2, float())
        bilip.setData(tmp)
        self.assertFalse(bilip.empty())

        tmp = bilip.getData()
        tmp.resize(0, 0, float())
        bilip.setData(tmp)
        self.assertTrue(bilip.empty())

        tmp = bilip.getData()
        tmp.resize(2, 2, float())
        bilip.setData(tmp)
        self.assertFalse(bilip.empty())

        tmp = bilip.getData()
        tmp.clear()
        bilip.setData(tmp)
        self.assertTrue(bilip.empty())

    def test_addValue(self):
        for i in range(-50, 101):
            p = i / 10.0

            for j in range(-50, 101):
                q = j / 10.0

                bilip_small = pyopenms.BilinearInterpolation()
                tmp = bilip_small.getData()
                tmp.resize(5, 5, float())
                bilip_small.setData(tmp)
                bilip_small.setMapping_0(0.0, 0.0, 5.0, 5.0)
                bilip_small.setMapping_1(0.0, 0.0, 5.0, 5.0)
                bilip_small.addValue(p, q, 100)

                bilip_big = pyopenms.BilinearInterpolation()
                tmp = bilip_big.getData()
                tmp.resize(15, 15, float())
                bilip_big.setData(tmp)
                bilip_big.setMapping_0(5.0, 0.0, 10.0, 5.0)
                bilip_big.setMapping_1(5.0, 0.0, 10.0, 5.0)
                bilip_big.addValue(p, q, 100)

                big_submatrix = pyopenms.MatrixDouble()
                big_submatrix.resize(5, 5, float())
                for m in range(5):
                    for n in range(5):
                        big_submatrix.setValue(m, n,
                                               bilip_big.getData().getValue(m+5,
                                                                            n+5))

                for m in range(bilip_small.getData().rows()):
                    for n in range(bilip_small.getData().cols()):
                        self.assertAlmostEqual(bilip_small.getData().getValue(m, n),
                                               big_submatrix.getValue(m, n))

    def test_value(self):
        bilip_small = pyopenms.BilinearInterpolation()
        bilip_big = pyopenms.BilinearInterpolation()

        tmp_small = bilip_small.getData()
        tmp_big = bilip_big.getData()

        tmp_small.resize(5, 5, float())
        tmp_big.resize(15, 15, float())

        for i in range(5):
            for j in range(5):
                num = int(math.floor(random.random() * 100))
                tmp_small.setValue(i, j, num)
                tmp_big.setValue(i + 5, j + 5, num)

        bilip_small.setData(tmp_small)
        bilip_big.setData(tmp_big)

        bilip_small.setMapping_0(0.0, 0.0, 5.0, 5.0)
        bilip_small.setMapping_1(0.0, 0.0, 5.0, 5.0)
        bilip_big.setMapping_0(5.0, 0.0, 10.0, 5.0)
        bilip_big.setMapping_1(5.0, 0.0, 10.0, 5.0)

        for i in range(-50, 101):
            p = i / 10.0
            for j in range(-50, 101):
                q = j / 10.0
                self.assertAlmostEqual(bilip_small.value(p, q),
                                       bilip_big.value(p, q))

if __name__ == '__main__':
    unittest.main()



from libcpp cimport bool
from Types cimport *
from DPosition cimport *

cdef extern from "<OpenMS/KERNEL/ChromatogramPeak.h>" namespace "OpenMS::ChromatogramPeak":

    ctypedef double IntensityType
    ctypedef double CoordinateType
    ctypedef DPosition1 PositionType

cdef extern from "<OpenMS/KERNEL/ChromatogramPeak.h>" namespace "OpenMS":

    cdef cppclass ChromatogramPeak:

        ChromatogramPeak() nogil except +
        ChromatogramPeak(ChromatogramPeak &) nogil except +
        ChromatogramPeak(PositionType retention_time, IntensityType intensity) nogil except +
        bool operator==(ChromatogramPeak) nogil except +
        bool operator!=(ChromatogramPeak) nogil except +

        # We will not catch C++ exceptions for get/set methods for performance
        # reasons (no memory allocation is involved).

        IntensityType getIntensity() nogil 
        void setIntensity(IntensityType) nogil 

        DPosition1 getPosition() nogil 
        void setPosition(DPosition1) nogil 

        CoordinateType getRT() nogil 
        void setRT(CoordinateType) nogil 

        CoordinateType getPos() nogil 
        void setPos(CoordinateType) nogil 

        # alias for getRT 
        CoordinateType getMZ() nogil 
        void setMZ(CoordinateType) nogil 




from Types cimport *
from IsobaricQuantitationMethod cimport *
from ItraqFourPlexQuantitationMethod cimport *
from ItraqEightPlexQuantitationMethod cimport *
from TMTSixPlexQuantitationMethod cimport *
from TMTTenPlexQuantitationMethod cimport *
from DefaultParamHandler cimport *
from MSExperiment cimport *
from ConsensusMap cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/IsobaricChannelExtractor.h>" namespace "OpenMS":
    
    cdef cppclass IsobaricChannelExtractor(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler

        IsobaricChannelExtractor(IsobaricChannelExtractor) nogil except +

        # IsobaricChannelExtractor(IsobaricQuantitationMethod *quant_method) nogil except + #wrap-ignore
        IsobaricChannelExtractor(ItraqEightPlexQuantitationMethod *quant_method) nogil except +
        IsobaricChannelExtractor(ItraqFourPlexQuantitationMethod *quant_method) nogil except +
        IsobaricChannelExtractor(TMTSixPlexQuantitationMethod *quant_method) nogil except +
        IsobaricChannelExtractor(TMTTenPlexQuantitationMethod *quant_method) nogil except +

        void extractChannels(MSExperiment & ms_exp_data, ConsensusMap & consensus_map) nogil except +




from ProgressLogger cimport *
from DefaultParamHandler cimport *
from MSExperiment cimport *
from MSSpectrum cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *
from libcpp.vector cimport vector as libcpp_vector

cdef extern from "<OpenMS/FILTERING/SMOOTHING/LowessSmoothing.h>" namespace "OpenMS":

    cdef cppclass LowessSmoothing(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler

        LowessSmoothing()      nogil except +
        LowessSmoothing(LowessSmoothing)      nogil except +

        void smoothData(libcpp_vector[double] x,
                        libcpp_vector[double] y,
                        libcpp_vector[double] & y_smoothed)      nogil except +




import unittest
import os

import pyopenms

class TestDIAScoring(unittest.TestCase):

    def setUp(self):
        pass

    def test_spectrum(self):

          intensity = [
            100, 100, 100, 100,
            100, 100, 100]
          mz = [
                #// four of the naked b/y ions 
                #// as well as one of the modified b and y ions ion each
                350.17164, #// b
                421.20875, #// b
                421.20875 + 79.9657, #// b + P
                547.26291, #// y
                646.33133, #// y
                809.39466 + 79.9657 #// y + P
          ]

          spectrum = pyopenms.OSSpectrum()
          spectrum.setMZArray(mz)
          spectrum.setIntensityArray(intensity)

          diascoring = pyopenms.DIAScoring()
          # diascoring.set_dia_parameters(0.05, False, 30, 50, 4, 4) // here we use a large enough window so that none of our peaks falls out
          p_dia = diascoring.getDefaults();
          p_dia.setValue("dia_extraction_window", 0.05);
          p_dia.setValue("dia_extraction_unit", "Th");
          p_dia.setValue("dia_centroided", "false");
          p_dia.setValue("dia_byseries_intensity_min", 30.0);
          p_dia.setValue("dia_byseries_ppm_diff", 50.0);
          p_dia.setValue("dia_nr_isotopes", 4);
          p_dia.setValue("dia_nr_charges", 4);
          diascoring.setParameters(p_dia);

          a = pyopenms.AASequence.fromString(b"SYVAWDR")

          bseries_score = 0.0
          yseries_score = 0.0
          charge = 1
          bseries_score, yseries_score = diascoring.dia_by_ion_score(spectrum, a, charge, bseries_score, yseries_score)

          self.assertAlmostEqual(bseries_score, 2.0)
          self.assertAlmostEqual(yseries_score, 2.0)

          # // now add a modification to the sequence
          a.setModification(1, b"Phospho" ) #; // modify the Y
          bseries_score = 0
          yseries_score = 0
          bseries_score, yseries_score = diascoring.dia_by_ion_score(spectrum, a, 1, bseries_score, yseries_score) 

          self.assertAlmostEqual (bseries_score, 1.0)
          self.assertAlmostEqual (yseries_score, 3.0)

if __name__ == '__main__':
    unittest.main()



from Types cimport *

cdef extern from "<OpenMS/FILTERING/DATAREDUCTION/SplinePackage.h>" namespace "OpenMS":
    
    cdef cppclass SplinePackage "OpenMS::SplinePackage":

        SplinePackage(libcpp_vector[double] pos, libcpp_vector[double] intensity) nogil except +
        SplinePackage(SplinePackage) nogil except + #wrap-ignore

        double getPosMin() nogil except +
        double getPosMax() nogil except +
        double getPosStepWidth() nogil except +
        bool isInPackage(double pos) nogil except +
        double eval(double pos) nogil except +




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from FeatureGroupingAlgorithm cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/FeatureGroupingAlgorithmLabeled.h>" namespace "OpenMS":
    
    cdef cppclass FeatureGroupingAlgorithmLabeled(FeatureGroupingAlgorithm) :
        # wrap-inherits:
        #  FeatureGroupingAlgorithm
        FeatureGroupingAlgorithmLabeled() nogil except +
        FeatureGroupingAlgorithmLabeled(FeatureGroupingAlgorithmLabeled) nogil except + #wrap-ignore
        void group(libcpp_vector[ FeatureMap ] & maps, ConsensusMap & out) nogil except +
        # POINTER # FeatureGroupingAlgorithm * create() nogil except +
        String getProductName() nogil except +




from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from DataValue cimport *
from String cimport *
from Types cimport *
from MetaInfoDescription cimport *

# see ../addons/FloatDataArray.pyx

cdef extern from "<OpenMS/METADATA/DataArrays.h>" namespace "OpenMS::DataArrays":

    cdef cppclass FloatDataArray(MetaInfoDescription):
        # wrap-inherits:
        #  MetaInfoDescription
        #
        # wrap-doc:
        #   The representation of extra float data attached to a spectrum or chromatogram.
        #   Raw data access is proved by `get_peaks` and `set_peaks`, which yields numpy arrays

        FloatDataArray() nogil except +
        FloatDataArray(FloatDataArray) nogil except + #wrap-ignore

        Size size() nogil except +
        void resize(size_t n) nogil except +
        void reserve(size_t n) nogil except +
        float& operator[](int) nogil except + # wrap-ignore
        void clear() nogil except +
        void push_back(float) nogil except +

        libcpp_vector[float].iterator begin() nogil # wrap-ignore
        libcpp_vector[float].iterator end()   nogil # wrap-ignore
        void assign(float*, float*) # wrap-ignore





from Types cimport *
from libcpp cimport bool
from Types cimport *
from String cimport *
from InterfaceDataStructures cimport *
# from ISpectrumAccess cimport *
from IndexedMzMLDecoder cimport *
from MSSpectrum cimport *
from MSChromatogram cimport *

cdef extern from "<OpenMS/FORMAT/HANDLERS/IndexedMzMLHandler.h>" namespace "OpenMS":
    
    cdef cppclass IndexedMzMLHandler "OpenMS::Internal::IndexedMzMLHandler":
        IndexedMzMLHandler() nogil except +
        IndexedMzMLHandler(IndexedMzMLHandler) nogil except +
        IndexedMzMLHandler(String filename) nogil except +

        void openFile(String filename) nogil except +
        bool getParsingSuccess() nogil except +

        size_t getNrSpectra() nogil except +
        size_t getNrChromatograms() nogil except +

        shared_ptr[Spectrum] getSpectrumById(int id_) nogil except +
        shared_ptr[Chromatogram] getChromatogramById(int id_) nogil except +

        MSSpectrum getMSSpectrumById(int id_) nogil except +
        void getMSSpectrumByNativeId(libcpp_string id_, MSSpectrum& spec) nogil except +
        MSChromatogram getMSChromatogramById(int id_) nogil except +
        void getMSChromatogramByNativeId(libcpp_string id_, MSChromatogram& chrom) nogil except +

        void setSkipXMLChecks(bool skip) nogil except +




from libcpp.vector cimport vector as libcpp_vector
from libcpp.set cimport set as libcpp_set
from String cimport *
from StringList cimport *
from ModificationDefinition cimport *
from PeptideIdentification cimport *
from AASequence cimport *

cdef extern from "<OpenMS/CHEMISTRY/ModificationDefinitionsSet.h>" namespace "OpenMS":

  cdef cppclass ModificationDefinitionsSet:

    ModificationDefinitionsSet() nogil except +
    ModificationDefinitionsSet(ModificationDefinitionsSet rhs) nogil except +

    # detailed constructor with StringLists
    # The StringLists should contain UniMod (www.unimod.org) terms, e.g. 'Carbamidomethyl (C)' or 'Oxidation (M)'
    ModificationDefinitionsSet(StringList fixed_modifications, StringList variable_modifications) nogil except +

    void setMaxModifications(Size max_mod) nogil except +
    Size getMaxModifications() nogil except +
    Size getNumberOfModifications() nogil except +
    Size getNumberOfFixedModifications() nogil except +
    Size getNumberOfVariableModifications() nogil except +
    void addModification(ModificationDefinition &mod_def) nogil except +
    void setModifications(libcpp_set[ ModificationDefinition ] &mod_defs) nogil except +
    void setModifications(const String &fixed_modifications, String &variable_modifications) nogil except +
    void setModifications(StringList &fixed_modifications, StringList &variable_modifications) nogil except +
    libcpp_set[ModificationDefinition] getModifications() nogil except +
    libcpp_set[ModificationDefinition] getFixedModifications() nogil except +
    libcpp_set[ModificationDefinition] getVariableModifications() nogil except +
    void getModificationNames(StringList &fixed_modifications, StringList &variable_modifications) nogil except +
    libcpp_set[String] getFixedModificationNames() nogil except +
    libcpp_set[String] getVariableModificationNames() nogil except +
    libcpp_set[String] getModificationNames() nogil except +
    bool isCompatible(AASequence &peptide) nogil except +
    void inferFromPeptides(libcpp_vector[ PeptideIdentification ] &peptides) nogil except +
    



from Types cimport *
from String cimport *
from MzTab cimport *

cdef extern from "<OpenMS/FORMAT/DATAACCESS/CsiFingerIdMzTabWriter.h>" namespace "OpenMS":
    
    cdef cppclass CsiFingerIdMzTabWriter "OpenMS::CsiFingerIdMzTabWriter":
        CsiFingerIdMzTabWriter() nogil except +  # wrap-ignore
        CsiFingerIdMzTabWriter(CsiFingerIdMzTabWriter) nogil except + #wrap-ignore

#
# wrap static method:
#
cdef extern from "<OpenMS/FORMAT/DATAACCESS/CsiFingerIdMzTabWriter.h>" namespace "OpenMS::CsiFingerIdMzTabWriter":

   void read(libcpp_vector[String] sirius_output_paths, String original_input_mzml, Size top_n_hits, MzTab & result) nogil except + # wrap-attach:CsiFingerIdMzTabWriter




from Types cimport *
from ConsensusIDAlgorithmIdentity cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/ConsensusIDAlgorithmWorst.h>" namespace "OpenMS":
    
    cdef cppclass ConsensusIDAlgorithmWorst(ConsensusIDAlgorithmIdentity) :
        # wrap-inherits:
        #  ConsensusIDAlgorithmIdentity
        ConsensusIDAlgorithmWorst() nogil except +
        ConsensusIDAlgorithmWorst(ConsensusIDAlgorithmWorst) nogil except + #wrap-ignore




from libcpp cimport bool
from Map cimport *
from String cimport *
from MetaInfoInterface cimport *
from CVTerm cimport *
from CVMappingRule cimport *
from ControlledVocabulary cimport *

cdef extern from "<OpenMS/METADATA/CVTermList.h>" namespace "OpenMS":

    cdef cppclass CVTermList(MetaInfoInterface):
        # wrap-inherits:
        #    MetaInfoInterface

        ######################################################################
        # Cython has a problem with inheritance of overloaded methods, so we
        # can only declare one of the two methods here.
        # see eg Precursor.pxd 

        CVTermList()            nogil except +
        CVTermList(CVTermList)  nogil except +

        void setCVTerms(libcpp_vector[CVTerm] & terms)  nogil except +
        void replaceCVTerm(CVTerm & term)               nogil except +

        # will not wrap due to Cython inheritance issue
        void replaceCVTerms(libcpp_vector[CVTerm] cv_terms, String accession) nogil except +
        # void replaceCVTerms(Map[String, libcpp_vector[CVTerm] ] cv_term_map) nogil except +

        void consumeCVTerms(Map[String, libcpp_vector[CVTerm] ] cv_term_map) nogil except +

        Map[String, libcpp_vector[CVTerm] ] getCVTerms() nogil except +
        void addCVTerm(CVTerm & term)                   nogil except +

        bool operator==(CVTermList)  nogil except +
        bool operator!=(CVTermList)  nogil except +

        bool hasCVTerm(String accession)  nogil except +
        bool empty()                      nogil except +




from libcpp.vector cimport vector as libcpp_vector
from Types cimport *
from ConsensusMap cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/ConsensusMapNormalizerAlgorithmQuantile.h>" namespace "OpenMS":

    cdef cppclass ConsensusMapNormalizerAlgorithmQuantile:

        ConsensusMapNormalizerAlgorithmQuantile() nogil except +
        ConsensusMapNormalizerAlgorithmQuantile(ConsensusMapNormalizerAlgorithmQuantile) nogil except + #wrap-ignore

        void normalizeMaps(ConsensusMap & input_map) nogil except +

        void resample(libcpp_vector[ double ] & data_in, libcpp_vector[ double ] & data_out, UInt n_resampling_points) nogil except +
        void extractIntensityVectors(ConsensusMap & map_, libcpp_vector[ libcpp_vector[ double ] ] & out_intensities) nogil except +
        void setNormalizedIntensityValues(libcpp_vector[ libcpp_vector[ double ] ] & feature_ints, ConsensusMap & map_) nogil except +




from Types cimport *
from String cimport *
from NASequence cimport *
from IdentificationData cimport *

cdef extern from "<OpenMS/CHEMISTRY/RNaseDigestion.h>" namespace "OpenMS":

    cdef cppclass RNaseDigestion:

      RNaseDigestion() nogil except +
      RNaseDigestion(RNaseDigestion) nogil except + # wrap-ignore

      SignedSize getMissedCleavages() nogil except +
      void setMissedCleavages(SignedSize missed_cleavages) nogil except +

      String getEnzymeName() nogil except +
      void setEnzyme(String name) nogil except +

#      void digest(const String & rna, libcpp_vector[ String ] & output, Size min_length, Size max_length) nogil except +
      void digest(NASequence & rna, libcpp_vector[ NASequence ] & output) nogil except +
      void digest(NASequence & rna, libcpp_vector[ NASequence ] & output, Size min_length, Size max_length) nogil except +

      void digest(IdentificationData & id_data) nogil except +
      void digest(IdentificationData & id_data, Size min_length, Size max_length) nogil except +

      # Returns the specificity for the digestion
      Specificity getSpecificity() nogil except +
      # Sets the specificity for the digestion (default is SPEC_FULL).
      void setSpecificity(Specificity spec) nogil except +
      # convert spec string name to enum
      Specificity getSpecificityByName(String name) nogil except +

      bool isValidProduct(String protein, Size pep_pos, Size pep_length,
                          bool ignore_missed_cleavages) nogil except +

      # void digestUnmodified(StringView sequence, libcpp_vector[ StringView ] & output, Size min_length, Size max_length) nogil except +

cdef extern from "<OpenMS/CHEMISTRY/RNaseDigestion.h>" namespace "OpenMS::RNaseDigestion":

    # protected
    # cdef cppclass BindingSite:
    #   BindingSite()
    #   Size position
    #   String AAname

    # cdef cppclass CleavageModel:
    #   CleavageModel()
    #   double p_cleave
    #   double p_miss

    cdef enum Specificity:
        # wrap-attach:
        #    RNaseDigestion
        SPEC_NONE = 0, # no requirements on start / end
        SPEC_SEMI = 1, # semi specific, i.e., one of the two cleavage sites must fulfill requirements
        SPEC_FULL = 2, # fully enzyme specific, e.g., tryptic (ends with KR, AA-before is KR), or peptide is at protein terminal ends
        SPEC_UNKNOWN = 3
        SPEC_NOCTERM = 8, # no requirements on CTerm (currently not supported in the class)
        SPEC_NONTERM = 9, # no requirements on NTerm (currently not supported in the class)
        SIZE_OF_SPECIFICITY = 10



from Types cimport *

cdef extern from "<OpenMS/MATH/MISC/RANSACModelQuadratic.h>" namespace "OpenMS::Math":

    cdef cppclass RansacModelQuadratic:
       RansacModelQuadratic() nogil except +
       RansacModelQuadratic(RansacModelQuadratic &) nogil except + # wrap-ignore




# two empty lines are important !


    def asDict(self):
        return dict(self.items())

    def keys(self):
        keys = list()
        cdef _ParamIterator param_it = self.inst.get().begin()
        while param_it != self.inst.get().end():
            key = param_it.getName().c_str()
            keys.append(key)
            inc(param_it)
        return keys

    def items(self):
        return [(k, self[k]) for k in self.keys()]

    def values(self):
        return [self[k] for k in self.keys()]

    def update(self, *a):
        """
        use cases:

           p.update(dict d)
           p.update(Param p)
           p.update(Param p, int flag)
        """

        cdef Param p
        cdef int flag
        cdef bool r

        if len(a) == 1 and isinstance(a[0], dict):
            dd, = a
            for key, v in dd.items():
                self[key] = v
            return True
        elif len(a) == 1 and isinstance(a[0], Param):
            p, = a
            r = self.inst.get().update(<_Param>deref(p.inst.get()))
            return r
        elif len(a) == 2 and isinstance(a[0], Param) and isinstance(a[1], int):
            p, flag = a
            r = self.inst.get().update(<_Param>deref(p.inst.get()), <bool> flag)
            return r
        else:
            raise Exception("can not handle parameters of type %s" % (map(type, a)))

    def get(self, bytes key, default=None):
        if self.exists(key):
            return self.getValue(key)
        return default

    def __getitem__(self, bytes key):
        return self.getValue(key)

    def __setitem__(self, bytes key, value):
        tags = self.getTags(key)
        desc = self.getDescription(key)
        self.setValue(key, value, desc, tags)













    ## def __iter__(self):
    ##     _r = self.inst.get().getSequence()
    ##     cdef libcpp_vector[const _Ribonucleotide *].iterator it__r = _r.begin()
    ##     cdef Ribonucleotide item_py_result
    ##     while it__r != _r.end():
    ##        item_py_result = Ribonucleotide.__new__(Ribonucleotide)
    ##        item_py_result.inst = shared_ptr[_Ribonucleotide](new _Ribonucleotide(deref(deref(it__r))))
    ##        yield py_result
    ##        inc(it__r)

    def __iter__(self):

        for r in self.getSequence():
          yield r




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from TargetedExperiment cimport *
from FASTAFile cimport *
from IntList cimport *
from PeptideIdentification cimport *
from FeatureMap cimport *

cdef extern from "<OpenMS/ANALYSIS/TARGETED/InclusionExclusionList.h>" namespace "OpenMS":

    cdef cppclass InclusionExclusionList(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        InclusionExclusionList() nogil except +
        InclusionExclusionList(InclusionExclusionList) nogil except + #wrap-ignore
        void writeTargets(libcpp_vector[ FASTAEntry ] & fasta_entries, const String & out_path, IntList & charges, const String rt_model_path) nogil except +
        void writeTargets(FeatureMap & map_, const String & out_path) nogil except +
        void writeTargets(libcpp_vector[ PeptideIdentification ] & pep_ids, const String & out_path, IntList & charges) nogil except +




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from DefaultParamHandler cimport *
from PeptideIdentification cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/ConsensusIDAlgorithm.h>" namespace "OpenMS":
    
    cdef cppclass ConsensusIDAlgorithm(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        # wrap-ignore
        # ABSTRACT class
        # no-pxd-import
        ConsensusIDAlgorithm() nogil except + #wrap-ignore
        ConsensusIDAlgorithm(ConsensusIDAlgorithm) nogil except + #wrap-ignore
        void apply(libcpp_vector[ PeptideIdentification ] & ids, Size number_of_runs) nogil except +




from Types cimport *
from String cimport *
from Ribonucleotide cimport *

cdef extern from "<OpenMS/CHEMISTRY/RibonucleotideDB.h>" namespace "OpenMS":
    
    cdef cppclass RibonucleotideDB "OpenMS::RibonucleotideDB":
        # wrap-manual-memory:
        #   cdef AutowrapPtrHolder[_RibonucleotideDB] inst

        RibonucleotideDB(RibonucleotideDB) nogil except + #wrap-ignore

        const Ribonucleotide * getRibonucleotide(const libcpp_string& code) nogil except +
        const Ribonucleotide * getRibonucleotidePrefix(const libcpp_string& code) nogil except +
        libcpp_pair[const Ribonucleotide *, const Ribonucleotide *] getRibonucleotideAlternatives(const libcpp_string& code) nogil except + # wrap-ignore

# COMMENT: wrap static methods
cdef extern from "<OpenMS/CHEMISTRY/RibonucleotideDB.h>" namespace "OpenMS::RibonucleotideDB":
    
    RibonucleotideDB* getInstance() nogil except + # wrap-ignore




from Types cimport *
from String cimport *

from streampos cimport *


cdef extern from "<OpenMS/FORMAT/HANDLERS/IndexedMzMLDecoder.h>" namespace "OpenMS":

    cdef cppclass IndexedMzMLDecoder:
        IndexedMzMLDecoder() nogil except +
        IndexedMzMLDecoder(IndexedMzMLDecoder) nogil except +
        int parseOffsets(String in_, int indexoffset, 
                libcpp_vector[ libcpp_pair[ libcpp_string, streampos] ]& spectra_offsets,
                libcpp_vector[ libcpp_pair[ libcpp_string, streampos] ]& chromatograms_offsets) nogil except + #wrap-ignore
        streampos findIndexListOffset(String in_, int buffersize) nogil except +



from Types cimport *
from libcpp cimport bool
from MSSpectrum cimport *

cdef extern from "<OpenMS/ANALYSIS/RNPXL/MorpheusScore.h>" namespace "OpenMS":
    
    cdef cppclass MorpheusScore "OpenMS::MorpheusScore":
        MorpheusScore(MorpheusScore) nogil except + #wrap-ignore
        MorpheusScore_Result compute(double fragment_mass_tolerance,
                                     bool fragment_mass_tolerance_unit_ppm,
                                     const MSSpectrum & exp_spectrum,
                                     const MSSpectrum & theo_spectrum) nogil except +

cdef extern from "<OpenMS/ANALYSIS/RNPXL/MorpheusScore.h>" namespace "OpenMS::MorpheusScore":
    
    cdef cppclass MorpheusScore_Result "OpenMS::MorpheusScore::Result":
        MorpheusScore_Result() nogil except + 
        MorpheusScore_Result(MorpheusScore_Result) nogil except + #wrap-ignore
        Size matches
        Size n_peaks
        float score
        float MIC
        float TIC
        float err




from String cimport *
from IonSource cimport *
from MassAnalyzer cimport *
from Software cimport *
from IonDetector cimport *
from MetaInfoInterface cimport *
from libcpp.vector cimport vector as libcpp_vector

cdef extern from "<OpenMS/METADATA/Instrument.h>" namespace "OpenMS":

    cdef cppclass Instrument(MetaInfoInterface):
        # wrap-inherits:
        #    MetaInfoInterface

        Instrument() nogil except +
        Instrument(Instrument) nogil except + # wrap-ignore

        # returns the name of the instrument
        String getName() nogil except +
        # sets the name of the instrument
        void setName(String name) nogil except +

        # returns the instrument vendor
        String getVendor() nogil except +
        # sets the instrument vendor
        void setVendor(String vendor) nogil except +

        # returns the instrument model
        String getModel() nogil except +
        # sets the instrument model
        void setModel(String model) nogil except +

        # returns a description of customizations
        String getCustomizations() nogil except +
        # sets the a description of customizations
        void setCustomizations(String customizations) nogil except +

        # returns a mutable reference to the ion source list
        libcpp_vector[IonSource] getIonSources() nogil except +
        # sets the ion source list
        void setIonSources(libcpp_vector[IonSource] ion_sources) nogil except +

        # returns a mutable reference to the mass analyzer list
        libcpp_vector[MassAnalyzer] getMassAnalyzers() nogil except +
        # sets the mass analyzer list
        void setMassAnalyzers(libcpp_vector[MassAnalyzer] mass_analyzers) nogil except +

        # returns a mutable reference to the ion detector list
        libcpp_vector[IonDetector] getIonDetectors() nogil except +
        # sets the ion detector list
        void setIonDetectors(libcpp_vector[IonDetector] ion_detectors) nogil except +

        # returns a mutable reference to the instrument software
        Software getSoftware() nogil except +
        # sets the instrument software
        void setSoftware(Software software) nogil except +

        IonOpticsType getIonOptics() nogil except +
        void setIonOptics(IonOpticsType ion_optics) nogil except +

cdef extern from "<OpenMS/METADATA/Instrument.h>" namespace "OpenMS::Instrument":

    cdef enum IonOpticsType:
    
        UNKNOWN,                          #< unknown
        MAGNETIC_DEFLECTION,              #< magnetic deflection
        DELAYED_EXTRACTION,               #< delayed extraction
        COLLISION_QUADRUPOLE,             #< collision quadrupole
        SELECTED_ION_FLOW_TUBE,           #< selected ion flow tube
        TIME_LAG_FOCUSING,                #< time lag focusing
        REFLECTRON,                       #< reflectron
        EINZEL_LENS,                      #< einzel lens
        FIRST_STABILITY_REGION,           #< first stability region
        FRINGING_FIELD,                   #< fringing field
        KINETIC_ENERGY_ANALYZER,          #< kinetic energy analyzer
        STATIC_FIELD,                     #< static field
        SIZE_OF_IONOPTICSTYPE




from Types cimport *
from TransformationModel cimport *
from TransformationModelInterpolated cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/TransformationModelLowess.h>" namespace "OpenMS":

    cdef cppclass TransformationModelLowess(TransformationModel) :
        # wrap-inherits:
        #  TransformationModel

        TransformationModelLowess() nogil except + #wrap-ignore
        TransformationModelLowess(TransformationModelLowess) nogil except + #wrap-ignore
        TransformationModelLowess(libcpp_vector[TM_DataPoint]& data, Param& params) nogil except +

        double evaluate(double value) nogil except +

# COMMENT: wrap static methods
cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/TransformationModelLowess.h>" namespace "OpenMS::TransformationModelLowess":     
    void getDefaultParameters(Param& params) nogil except + #wrap-attach:TransformationModelLowess



from Types cimport *
from libcpp cimport bool
from MapAlignmentEvaluationAlgorithm cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/MapAlignmentEvaluationAlgorithmRecall.h>" namespace "OpenMS":
    
    cdef cppclass MapAlignmentEvaluationAlgorithmRecall(MapAlignmentEvaluationAlgorithm) :
        # wrap-inherits:
        #  MapAlignmentEvaluationAlgorithm
        MapAlignmentEvaluationAlgorithmRecall() nogil except +
        MapAlignmentEvaluationAlgorithmRecall(MapAlignmentEvaluationAlgorithmRecall) nogil except + #wrap-ignore

        # NAMESPACE # void evaluate(ConsensusMap & consensus_map_in, ConsensusMap & consensus_map_gt, double & rt_dev, double & mz_dev, Peak2D::IntensityType & int_dev, bool use_charge, double & out) nogil except +
        # POINTER # MapAlignmentEvaluationAlgorithm * create() nogil except +
        String getProductName() nogil except +




from Types cimport *

cdef extern from "<OpenMS/OPENSWATHALGO/DATAACCESS/DataStructures.h>" namespace "OpenSwath":
    
    cdef cppclass OSSpectrumMeta "OpenSwath::OSSpectrumMeta":

        OSSpectrumMeta() nogil except +
        OSSpectrumMeta(OSSpectrumMeta) nogil except + #wrap-ignore

        size_t index
        libcpp_string id
        double RT
        int ms_level




from libcpp.vector cimport vector as libcpp_vector
from libcpp cimport bool
from Types cimport *
from Matrix cimport *

cdef extern from "<OpenMS/MATH/MISC/BilinearInterpolation.h>" namespace "OpenMS::Math":
    
    cdef cppclass BilinearInterpolation[KeyType,ValueType]:
        # wrap-instances:
        #   BilinearInterpolation := BilinearInterpolation[double, double]

        BilinearInterpolation() nogil except +
        BilinearInterpolation(BilinearInterpolation) nogil except +
        ValueType value(KeyType arg_pos_0, KeyType arg_pos_1) nogil except +
        void addValue(KeyType arg_pos_0, KeyType arg_pos_1, ValueType arg_value) nogil except +

        Matrix[ValueType] getData() nogil except +
        void setData(Matrix[ValueType] & data) nogil except +

        bool empty() nogil except +

        KeyType key2index_0(KeyType pos) nogil except +
        KeyType index2key_0(KeyType pos) nogil except +
        KeyType key2index_1(KeyType pos) nogil except +
        KeyType index2key_1(KeyType pos) nogil except +

        KeyType getScale_0() nogil except +
        void setScale_0(KeyType & scale) nogil except +
        KeyType getScale_1() nogil except +
        void setScale_1(KeyType & scale) nogil except +

        KeyType getOffset_0() nogil except +
        void setOffset_0(KeyType & offset) nogil except +
        KeyType getOffset_1() nogil except +
        void setOffset_1(KeyType & offset) nogil except +

        void setMapping_0(KeyType & scale, KeyType & inside, KeyType & outside) nogil except +
        void setMapping_0(KeyType & inside_low, KeyType & outside_low, KeyType & inside_high, KeyType & outside_high) nogil except +
        void setMapping_1(KeyType & scale, KeyType & inside, KeyType & outside) nogil except +
        void setMapping_1(KeyType & inside_low, KeyType & outside_low, KeyType & inside_high, KeyType & outside_high) nogil except +

        KeyType  getInsideReferencePoint_0() nogil except +
        KeyType  getInsideReferencePoint_1() nogil except +

        KeyType  getOutsideReferencePoint_0() nogil except +
        KeyType  getOutsideReferencePoint_1() nogil except +

        KeyType supportMin_0() nogil except +
        KeyType supportMin_1() nogil except +

        KeyType supportMax_0() nogil except +
        KeyType supportMax_1() nogil except +

        # BilinearInterpolation(KeyType scale, KeyType offset) nogil except +




import unittest
import os

import pyopenms

class TestChromatogramExtractorAlgorithm(unittest.TestCase):

    def setUp(self):
        dirname = os.path.dirname(os.path.abspath(__file__))
        self.filename = os.path.join(dirname, "test2.mzML").encode()

    def test_readfile_content(self):
        exp = pyopenms.MSExperiment()
        pyopenms.MzMLFile().load(self.filename, exp)
        exp_size = exp.size()
        saccess = pyopenms.SpectrumAccessOpenMS(exp)

        ### double mz # mz around which should be extracted
        ### double rt_start # rt start of extraction (in seconds)
        ### double rt_end # rt end of extraction (in seconds)
        ### libcpp_string id # identifier
        targeted = []
        coord = pyopenms.ExtractionCoordinates()
        coord.mz = 618.31
        coord.rt_start = 4000 
        coord.rt_end = 5000
        coord.id = b"tr3"
        targeted.append(coord)

        coord = pyopenms.ExtractionCoordinates()
        coord.mz = 628.45
        coord.rt_start = 4000 
        coord.rt_end = 5000
        coord.id = b"tr1"
        targeted.append(coord)

        coord = pyopenms.ExtractionCoordinates()
        coord.mz = 654.38
        coord.rt_start = 4000 
        coord.rt_end = 5000
        coord.id = b"tr2"
        targeted.append(coord)

        trafo = pyopenms.TransformationDescription()

        # Start with length zero
        tmp_out = [ pyopenms.OSChromatogram() for i in range(len(targeted))]
        self.assertEqual(len(tmp_out[0].getIntensityArray()), 0)

        extractor = pyopenms.ChromatogramExtractorAlgorithm()
        mz_extraction_window = 10.0
        ppm = False
        extractor.extractChromatograms(saccess, tmp_out, targeted, mz_extraction_window, ppm, -1.0, b"tophat")

        # Basically test that the output is non-zero (e.g. the data is
        # correctly relayed to python)
        # The functionality is not tested here!
        self.assertEqual(len(tmp_out), len(targeted))
        self.assertNotEqual(len(tmp_out), 0)

        # End with different length
        self.assertEqual(len(tmp_out[0].getIntensityArray()), exp_size)
        self.assertNotEqual(len(tmp_out[0].getIntensityArray()), 0)
        self.assertNotEqual(len(tmp_out[0].getTimeArray()), 0)
        self.assertNotEqual(tmp_out[0].getIntensityArray()[0], 0)
        self.assertNotEqual(tmp_out[0].getTimeArray()[0], 0)

if __name__ == '__main__':
    unittest.main()



from OPXLDataStructs cimport *
from ProteinProteinCrossLink cimport ProteinProteinCrossLink
from PeptideHit cimport PeptideHit_PeakAnnotation

cdef extern from "<OpenMS/ANALYSIS/XLMS/OPXLDataStructs.h>" namespace "OpenMS::OPXLDataStructs":

    cdef cppclass CrossLinkSpectrumMatch "OpenMS::OPXLDataStructs::CrossLinkSpectrumMatch":

        CrossLinkSpectrumMatch(CrossLinkSpectrumMatch) nogil except +
        CrossLinkSpectrumMatch() nogil except +

        ProteinProteinCrossLink cross_link

        Size scan_index_light
        Size scan_index_heavy
        double score
        Size rank
        double xquest_score
        double pre_score
        double percTIC
        double wTIC
        double wTICold
        double int_sum
        double intsum_alpha
        double intsum_beta
        double total_current
        double precursor_error_ppm

        double match_odds
        double match_odds_alpha
        double match_odds_beta
        double log_occupancy
        double log_occupancy_alpha
        double log_occupancy_beta
        double xcorrx_max
        double xcorrc_max

        Size matched_linear_alpha
        Size matched_linear_beta
        Size matched_xlink_alpha
        Size matched_xlink_beta

        double num_iso_peaks_mean
        double num_iso_peaks_mean_linear_alpha
        double num_iso_peaks_mean_linear_beta
        double num_iso_peaks_mean_xlinks_alpha
        double num_iso_peaks_mean_xlinks_beta

        double ppm_error_abs_sum_linear_alpha
        double ppm_error_abs_sum_linear_beta
        double ppm_error_abs_sum_xlinks_alpha
        double ppm_error_abs_sum_xlinks_beta
        double ppm_error_abs_sum_linear
        double ppm_error_abs_sum_xlinks
        double ppm_error_abs_sum_alpha
        double ppm_error_abs_sum_beta
        double ppm_error_abs_sum

        int precursor_correction

        double precursor_total_intensity
        double precursor_target_intensity
        double precursor_signal_proportion
        Size precursor_target_peak_count
        Size precursor_residual_peak_count

        libcpp_vector[ PeptideHit_PeakAnnotation ] frag_annotations
        Size peptide_id_index



from Types cimport *
from String cimport String

cdef extern from "<OpenMS/FORMAT/DATAACCESS/CsiFingerIdMzTabWriter.h>" namespace "OpenMS::CsiFingerIdMzTabWriter":
    
    cdef cppclass CsiAdapterHit "OpenMS::CsiFingerIdMzTabWriter::CsiAdapterHit":
        CsiAdapterHit() nogil except + 
        CsiAdapterHit(CsiAdapterHit) nogil except + #wrap-ignore
        String inchikey2D
        String inchi
        unsigned int rank
        String molecular_formula
        double score
        String name
        String smiles
        libcpp_vector[ String ] pubchemids
        libcpp_vector[ String ] links




# from InterpolationModel cimport *
# from BasicStatistics cimport *
from String cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/BiGaussModel.h>" namespace "OpenMS":
    
    cdef cppclass BiGaussModel "OpenMS::BiGaussModel":
        BiGaussModel() nogil except +
        BiGaussModel(BiGaussModel) nogil except +
        void setOffset(double offset) nogil except +
        void setSamples() nogil except +
        double getCenter() nogil except +
        # BaseModel[ 1 ] * create() nogil except +
        String getProductName() nogil except +




import sys

import pyopenms

"""
python pyTOPP/OpenSwathRTNormalizer.py --in ../source/TEST/TOPP/OpenSwathRTNormalizer_1_input.mzML \
        --tr ../source/TEST/TOPP/OpenSwathRTNormalizer_1_input.TraML --out OpenSwathRTNormalizer.tmp.out \
        && diff OpenSwathRTNormalizer.tmp.out ../source/TEST/TOPP/OpenSwathRTNormalizer_1_output.trafoXML

"""

def simple_find_best_feature(output, pairs, targeted):
  f_map = {}
  for f in output:
    key = f.getMetaValue("PeptideRef")
    if f_map.has_key(key):
      f_map[key].append(f)
    else:
      f_map[key] = [f]


  for v in f_map.values():
    bestscore = -10000
    for feature in v:
      score = feature.getMetaValue("main_var_xx_lda_prelim_score")
      if score > bestscore:
        best = feature
        bestscore = score

    pep = targeted.getPeptideByRef( feature.getMetaValue("PeptideRef")  )
    pairs.append( [best.getRT(), pep.getRetentionTime() ] )

def algorithm(chromatograms, targeted):
    # Create empty files as input and finally as output
    empty_swath = pyopenms.MSExperiment()
    trafo = pyopenms.TransformationDescription()
    output = pyopenms.FeatureMap();

    # set up featurefinder and run
    featurefinder = pyopenms.MRMFeatureFinderScoring()
    # set the correct rt use values
    scoring_params = pyopenms.MRMFeatureFinderScoring().getDefaults();
    scoring_params.setValue("Scores:use_rt_score",'false', '')
    featurefinder.setParameters(scoring_params);
    featurefinder.pickExperiment(chromatograms, output, targeted, trafo, empty_swath)

    # get the pairs
    pairs=[]
    simple_find_best_feature(output, pairs, targeted)
    pairs_corrected = pyopenms.MRMRTNormalizer().rm_outliers( pairs, 0.95, 0.6)
    pairs_corrected = [ list(p) for p in pairs_corrected]

    # // store transformation, using a linear model as default
    trafo_out = pyopenms.TransformationDescription()
    trafo_out.setDataPoints(pairs_corrected);
    model_params = pyopenms.Param()
    model_params.setValue("symmetric_regression", 'false', '');
    model_type = "linear";
    trafo_out.fitModel(model_type, model_params);
    return trafo_out

def main(options):

    # load chromatograms
    chromatograms = pyopenms.MSExperiment()
    fh = pyopenms.FileHandler()
    fh.loadExperiment(options.infile, chromatograms)

    # load TraML file
    targeted = pyopenms.TargetedExperiment();
    tramlfile = pyopenms.TraMLFile();
    tramlfile.load(options.traml_in, targeted);

    trafo_out = algorithm(chromatograms, targeted)

    pyopenms.TransformationXMLFile().store(options.outfile, trafo_out);

def handle_args():
    import argparse

    usage = ""
    usage += "\nMRMMapper maps measured chromatograms (mzML) and the transitions used (TraML)"

    parser = argparse.ArgumentParser(description = usage )
    parser.add_argument('--in', dest="infile", help = 'An input file containing chromatograms')
    parser.add_argument("--tr", dest="traml_in", help="TraML input file containt the transitions")
    parser.add_argument("--out", dest="outfile", help="Output trafoXML file with annotated chromatograms")

    args = parser.parse_args(sys.argv[1:])
    return args

if __name__ == '__main__':
    options = handle_args()
    main(options)



from libcpp.vector cimport vector as libcpp_vector
from IsotopeCluster cimport *
from DefaultParamHandler cimport *
from Feature cimport *
# from BasicStatistics cimport *
# from FeatureFinderDefs cimport *
from Peak1D cimport *

ctypedef libcpp_vector[Peak1D] RawDataArrayType

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/Fitter1D.h>" namespace "OpenMS":
    
    cdef cppclass Fitter1D(DefaultParamHandler):
        # wrap-inherits:
        #  DefaultParamHandler
        Fitter1D() nogil except +
        Fitter1D(Fitter1D) nogil except +
        # QualityType fit1d(RawDataArrayType &, InterpolationModel *&) nogil except +
        void registerChildren() nogil except +




from libcpp.vector cimport vector as libcpp_vector
from libcpp cimport bool

from String cimport *
from StringList cimport *
from CVMappings cimport *
from ControlledVocabulary cimport *

cdef extern from "<OpenMS/FORMAT/VALIDATORS/SemanticValidator.h>" namespace "OpenMS::Internal":

    cdef cppclass SemanticValidator:

        SemanticValidator(CVMappings mapping, ControlledVocabulary cv) nogil except +

        bool validate(String filename, StringList errors, StringList warnings) nogil except +

        # Checks if a CVTerm is allowed in a given path
        bool locateTerm(String path, SemanticValidator_CVTerm & parsed_term) nogil except +

        # Sets the CV parameter tag name (default: 'cvParam')
        void setTag(String tag) nogil except +

        # Sets the name of the attribute for accessions in the CV parameter tag name (default: 'accession')
        void setAccessionAttribute(String accession) nogil except +

        # Sets the name of the attribute for accessions in the CV parameter tag name (default: 'name')
        void setNameAttribute(String name) nogil except +

        # Sets the name of the attribute for accessions in the CV parameter tag name (default: 'value')
        void setValueAttribute(String value) nogil except +

        # Set if CV term value types should be check (enabled by default)
        void setCheckTermValueTypes(bool check) nogil except +

        # Set if CV term units should be check (disabled by default)
        void setCheckUnits(bool check) nogil except +

        # Sets the name of the unit accession attribute (default: 'unitAccession')
        void setUnitAccessionAttribute(String accession) nogil except +

        # Sets the name of the unit name attribute (default: 'unitName')
        void setUnitNameAttribute(String name) nogil except +

cdef extern from "<OpenMS/FORMAT/VALIDATORS/SemanticValidator.h>" namespace "OpenMS::Internal::SemanticValidator":

      cdef cppclass SemanticValidator_CVTerm "OpenMS::Internal::SemanticValidator::CVTerm":
        SemanticValidator_CVTerm() nogil except +
        SemanticValidator_CVTerm(SemanticValidator_CVTerm) nogil except +
        String accession
        String name
        String value
        bool has_value
        String unit_accession
        bool has_unit_accession
        String unit_name
        bool has_unit_name



from Types cimport *
from libcpp.set cimport set as libcpp_set
from MSExperiment cimport *
from PeakPickerHiRes cimport *
from ProgressLogger cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/PeakWidthEstimator.h>" namespace "OpenMS":
    
    cdef cppclass PeakWidthEstimator "OpenMS::PeakWidthEstimator":

        # PeakWidthEstimator() nogil except + # default constructor is private
        PeakWidthEstimator(PeakWidthEstimator) nogil except + #wrap-ignore
        PeakWidthEstimator(MSExperiment exp_picked,
                           libcpp_vector[libcpp_vector[PeakBoundary] ] & boundaries) nogil except +

        double getPeakWidth(double mz) nogil except +




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from Matrix cimport *

cdef extern from "<OpenMS/ANALYSIS/PIP/LocalLinearMap.h>" namespace "OpenMS":
    
    cdef cppclass LocalLinearMap "OpenMS::LocalLinearMap":
        LocalLinearMap() nogil except +
        LocalLinearMap(LocalLinearMap) nogil except + #wrap-ignore
        LLMParam getLLMParam() nogil except +
        Matrix[ double ]  getCodebooks() nogil except +
        Matrix[ double ]  getMatrixA() nogil except +
        libcpp_vector[ double ]  getVectorWout() nogil except +
        # TODO STL attributes unsigned int 
        # Matrix[ UInt ]  getCord() nogil except +
        void normalizeVector(libcpp_vector[ double ] & aaIndexVariables) nogil except +
        # libcpp_vector[ double ] neigh(Matrix[ unsigned int ] & cord, Size win, double radius) nogil except +


cdef extern from "<OpenMS/ANALYSIS/PIP/LocalLinearMap.h>" namespace "OpenMS::LocalLinearMap":
    
    cdef cppclass LLMParam "OpenMS::LocalLinearMap::LLMParam":
        LLMParam() nogil except +
        LLMParam(LLMParam) nogil except + #wrap-ignore
        UInt xdim
        UInt ydim
        double radius






    # C++ signature: void align(libcpp_vector[MSExperiment] &, libcpp_vector[TransformationDescription] &, int)
    # C++ signature: void align(libcpp_vector[FeatureMap] &, libcpp_vector[TransformationDescription] &, int)
    # C++ signature: void align(libcpp_vector[ConsensusMap] &, libcpp_vector[TransformationDescription] &, int)
    # 
    # In addition to above overloaded functions adding this new one.
    def align_4(self, list ids , list trafos, int ref_index):
        """
        Parameters:
        ids (list): list of lists of PeptideIdentification objects
        trafos (list): list of TransformationDescription objects
        ref_index (int)
        """
        assert isinstance(ids, list) and all(isinstance(li, list) and all(isinstance(li_, PeptideIdentification) for li_ in li) for li in ids), 'arg ids wrong type'
        assert isinstance(trafos, list) and all(isinstance(li, TransformationDescription) for li in trafos), 'arg trafos wrong type'
        cdef libcpp_vector[libcpp_vector[_PeptideIdentification]] * v0 = new libcpp_vector[libcpp_vector[_PeptideIdentification]]()
        cdef libcpp_vector[_PeptideIdentification] * v0_rec = new libcpp_vector[_PeptideIdentification]()
        cdef PeptideIdentification item0_rec
        cdef libcpp_vector[_PeptideIdentification].iterator it_ids_rec
        for ids_rec in ids:
            v0_rec.clear()
            for item0_rec in ids_rec:
                v0_rec.push_back(deref(item0_rec.inst.get()))
            v0.push_back(deref(v0_rec))
        cdef libcpp_vector[_TransformationDescription] * v1 = new libcpp_vector[_TransformationDescription]()
        cdef TransformationDescription item1
        for item1 in trafos:
            v1.push_back(deref(item1.inst.get()))
        self.inst.get().align(deref(v0), deref(v1), ref_index)
        cdef libcpp_vector[_TransformationDescription].iterator it_trafos = v1.begin()
        replace_0 = []
        while it_trafos != v1.end():
            item1 = TransformationDescription.__new__(TransformationDescription)
            item1.inst = shared_ptr[_TransformationDescription](new _TransformationDescription(deref(it_trafos)))
            replace_0.append(item1)
            inc(it_trafos)
        trafos[:] = replace_0
        del v1
        cdef libcpp_vector[libcpp_vector[_PeptideIdentification]].iterator it_ids = v0.begin()
        replace_0 = []
        while it_ids != v0.end():
            it_ids_rec = deref(it_ids).begin()
            replace_1 = []
            while it_ids_rec != deref(it_ids).end():
                item0_rec = PeptideIdentification.__new__(PeptideIdentification)
                item0_rec.inst = shared_ptr[_PeptideIdentification](new _PeptideIdentification(deref(it_ids_rec)))
                replace_1.append(item0_rec)
                inc(it_ids_rec)
            replace_0.append(replace_1)
            inc(it_ids)
        ids[:] = replace_0
        del v0




from Types cimport *
from DPosition cimport *
from libcpp cimport bool

cdef extern from "<OpenMS/DATASTRUCTURES/DRange.h>" namespace "OpenMS":
    
    cdef cppclass DRange1 "OpenMS::DRange<1> ":
        DRange1() nogil except +
        DRange1(DRange1) nogil except +
        DRange1(DPosition1 lower, DPosition1 upper) nogil except +
        bool operator==(DRange1 & rhs) nogil except +
        bool encloses(DPosition1 & position) nogil except +
        DRange1 united(DRange1 other_range) nogil except +
        # DRangeIntersection intersects(DRange1 & range_) nogil except +
        bool isIntersected(DRange1 & range_) nogil except +
        bool isEmpty() nogil except +

    cdef cppclass DRange2 "OpenMS::DRange<2> ":
        DRange2() nogil except +
        DRange2(DRange2) nogil except +
        DRange2(DPosition2 lower, DPosition2 upper) nogil except +
        DRange2(double minx, double miny, double maxx, double maxy) nogil except +
        bool operator==(DRange2 & rhs) nogil except +
        # bool encloses(DPosition2 & position) nogil except +
        # bool encloses(double x, double y) nogil except +
        DRange2 united(DRange2 other_range) nogil except +
        # DRangeIntersection intersects(DRange2 & range_) nogil except +
        bool isIntersected(DRange2 & range_) nogil except +
        bool isEmpty() nogil except +

cdef extern from "<OpenMS/DATASTRUCTURES/DRange.h>" namespace "OpenMS::DRange":
    cdef enum DRangeIntersection "OpenMS::DRange::DRangeIntersection":
        Disjoint
        Intersects
        Inside






    def __long__(self):
        # <long> calls c++ conversion streampos -> c++ long value
        # long(..) converts this value to a Python long.
        # value. else this method may return an int or long, depending on the magnitude of
        # the value !
        return long(<long>(deref(self.inst.get())))



from libcpp.vector cimport vector as libcpp_vector
from Types cimport *

cdef extern from "<OpenMS/DATASTRUCTURES/IsotopeCluster.h>" namespace "OpenMS":
    
    cdef cppclass IsotopeCluster "OpenMS::IsotopeCluster":
        IsotopeCluster() nogil except +
        IsotopeCluster(IsotopeCluster) nogil except + #wrap-ignore
        ChargedIndexSet peaks
        libcpp_vector[ size_t ] scans

cdef extern from "<OpenMS/DATASTRUCTURES/IsotopeCluster.h>" namespace "OpenMS::IsotopeCluster":
    
    cdef cppclass ChargedIndexSet "OpenMS::IsotopeCluster::ChargedIndexSet":
        ChargedIndexSet() nogil except +
        ChargedIndexSet(ChargedIndexSet) nogil except + #wrap-ignore
        Int charge




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from InterfaceDataStructures cimport *

cdef extern from "<OpenMS/FILTERING/NOISEESTIMATION/SignalToNoiseEstimatorMedianRapid.h>" namespace "OpenMS":
    
    cdef cppclass SignalToNoiseEstimatorMedianRapid "OpenMS::SignalToNoiseEstimatorMedianRapid":

        SignalToNoiseEstimatorMedianRapid(SignalToNoiseEstimatorMedianRapid) nogil except + #wrap-ignore
        SignalToNoiseEstimatorMedianRapid(double window_length) nogil except +
        NoiseEstimator estimateNoise(shared_ptr[Spectrum]) nogil except +
        NoiseEstimator estimateNoise(shared_ptr[Chromatogram]) nogil except +
        NoiseEstimator estimateNoise(libcpp_vector[ double ] mz_array, libcpp_vector[ double ] int_array) nogil except +

cdef extern from "<OpenMS/FILTERING/NOISEESTIMATION/SignalToNoiseEstimatorMedianRapid.h>" namespace "OpenMS::SignalToNoiseEstimatorMedianRapid":
    
    cdef cppclass NoiseEstimator "OpenMS::SignalToNoiseEstimatorMedianRapid::NoiseEstimator":

        NoiseEstimator() nogil except + #wrap-ignore
        NoiseEstimator(NoiseEstimator) nogil except + #wrap-ignore
        NoiseEstimator(double nr_windows_, double mz_start_, double win_len_) nogil except +

        int nr_windows
        double mz_start
        double window_length
        libcpp_vector[ double ] result_windows_even
        libcpp_vector[ double ] result_windows_odd

        double get_noise_value(double mz) nogil except +
        double get_noise_even(double mz) nogil except +
        double get_noise_odd(double mz) nogil except +




from String cimport *
from StringList cimport *
from TargetedExperiment cimport *

cdef extern from "<OpenMS/FORMAT/TraMLFile.h>" namespace "OpenMS":

    cdef cppclass TraMLFile:

        TraMLFile() nogil except +

        void load(String filename,
                  TargetedExperiment & id) nogil except +

        void store(String filename,
                  TargetedExperiment & id) nogil except +

        bool isSemanticallyValid(String filename, StringList & errors,
                                 StringList & warnings) nogil except +



from Types cimport *
from MSSpectrum cimport *
from MSExperiment cimport *

cdef extern from "<OpenMS/METADATA/SpectrumLookup.h>" namespace "OpenMS":

    cdef cppclass SpectrumLookup:

        SpectrumLookup() nogil except +
        # SpectrumLookup(SpectrumLookup) nogil except + # private

        double rt_tolerance

        bool empty() nogil except +

        void readSpectra(MSExperiment spectra, String scan_regexp) nogil except +

        Size findByRT(double rt) nogil except +

        Size findByNativeID(String native_id) nogil except +
        
        # This is a base class: we cannot overload methods since Cython doesn't
        # allow inheritance of overloaded methods.

        # Size findByIndex(Size index) nogil except +

        Size findByIndex(Size index, bool count_from_one) nogil except +

        Size findByScanNumber(Size scan_number) nogil except +

        Size findByReference(String spectrum_ref) nogil except +

        void addReferenceFormat(String regexp) nogil except +

        # # NAMESPACE # Int extractScanNumber(const String & native_id, boost::regex & scan_regexp, bool no_error) nogil except +

        Int extractScanNumber(const String& native_id, const String& native_id_type_accession) nogil except +




from Types cimport *
from MSExperiment cimport *
from PeptideIdentification cimport *
from ProteinIdentification cimport *
from SpectrumMetaDataLookup cimport *

cdef extern from "<OpenMS/FORMAT/PercolatorOutfile.h>" namespace "OpenMS":
    
    cdef cppclass PercolatorOutfile "OpenMS::PercolatorOutfile":
        PercolatorOutfile() nogil except +
        PercolatorOutfile(PercolatorOutfile) nogil except + #wrap-ignore

        # libcpp_string score_type_names()
        PercolatorOutfile_ScoreType getScoreType(String score_type_name) nogil except +

        void load(const String & filename, ProteinIdentification & proteins,
                  libcpp_vector[ PeptideIdentification ] & peptides,
                  SpectrumMetaDataLookup & lookup, 
                  PercolatorOutfile_ScoreType output_score) nogil except +

cdef extern from "<OpenMS/FORMAT/PercolatorOutfile.h>" namespace "OpenMS::PercolatorOutfile":
    cdef enum PercolatorOutfile_ScoreType "OpenMS::PercolatorOutfile::ScoreType":
        #wrap-attach:
        #    PercolatorOutfile
        QVALUE
        POSTERRPROB
        SCORE
        SIZE_OF_SCORETYPE




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from BaseGroupFinder cimport *
from ConsensusMap cimport *
from ProgressLogger cimport *
from DefaultParamHandler cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/SimplePairFinder.h>" namespace "OpenMS":
    
    cdef cppclass SimplePairFinder(BaseGroupFinder) :
        # wrap-inherits:
        #  BaseGroupFinder

        SimplePairFinder() nogil except +
        SimplePairFinder(SimplePairFinder) nogil except + #wrap-ignore

        void run(libcpp_vector[ ConsensusMap ] & input_maps, ConsensusMap & result_map) nogil except +
        # POINTER # BaseGroupFinder * create() nogil except +
        String getProductName() nogil except +



import unittest,os

import env
import pyopenms
from collections import defaultdict

eps = 2

def simple_find_best_feature(output, pairs, targeted):
    f_map = defaultdict(list)
    for f in output:
        key = f.getMetaValue("PeptideRef".encode())
        f_map[key].append(f)

    get_score = lambda f: f.getMetaValue("main_var_xx_lda_prelim_score".encode())
    for fl in f_map.values():
        scores = [(get_score(fi), fi)  for fi in fl]
        best_score, best_feature = max(scores)
        __, feature = scores[-1]

        pep = targeted.getPeptideByRef(
                            feature.getMetaValue("PeptideRef".encode())
                            )
        pairs.append([best_feature.getRT(), pep.getRetentionTime()])


class TestMRMRTNormalizer(unittest.TestCase):
    """Emulates the behavior of OpenSwathMRMRTNormalizer"""

    def setUp(self):
        self.testdirname = os.path.join(env.OPEN_MS_SRC, "src/tests/topp")
        # set up files
        self.chromatograms = os.path.join(self.testdirname, "OpenSwathRTNormalizer_1_input.mzML").encode()
        self.tramlfile = os.path.join(self.testdirname, "OpenSwathRTNormalizer_1_input.TraML").encode()

    def test_run_mrmrtnormalizer(self):

        # load chromatograms
        chromatograms = pyopenms.MSExperiment()
        fh = pyopenms.FileHandler()
        fh.loadExperiment(self.chromatograms, chromatograms)

        # load TraML file
        targeted = pyopenms.TargetedExperiment();
        tramlfile = pyopenms.TraMLFile();
        tramlfile.load(self.tramlfile, targeted);

        # Create empty files as input and finally as output
        empty_swath = pyopenms.MSExperiment()
        trafo = pyopenms.TransformationDescription()
        output = pyopenms.FeatureMap();

        # set up featurefinder and run
        featurefinder = pyopenms.MRMFeatureFinderScoring()
        # set the correct rt use values
        scoring_params = pyopenms.MRMFeatureFinderScoring().getDefaults();
        scoring_params.setValue("Scores:use_rt_score".encode(), 'false'.encode(), ''.encode())
        featurefinder.setParameters(scoring_params);
        featurefinder.pickExperiment(chromatograms, output, targeted, trafo, empty_swath)

        # get the pairs
        pairs=[]
        simple_find_best_feature(output, pairs, targeted)
        pairs_corrected = pyopenms.MRMRTNormalizer().removeOutliersIterative(pairs, 0.95, 0.6, True, b"iter_jackknife")
        pairs_corrected = [ list(p) for p in pairs_corrected]

        expected = [(1497.56884765625, 1881.0),
             (2045.9776611328125, 2409.0),
             (2151.4814453125, 2509.0),
             (1924.0750732421875, 2291.0),
             (612.9832153320312, 990.0),
             (1086.2474365234375, 1470.0),
             (1133.89404296875, 1519.0),
             (799.5291137695312, 1188.0),
             (1397.1541748046875, 1765.0)]

        for exp,res in zip(sorted(expected), sorted(pairs_corrected)):
            self.assertAlmostEqual(exp[0], res[0], eps)
            self.assertAlmostEqual(exp[1], res[1], eps)

if __name__ == '__main__':
    unittest.main()




from Types cimport *
from Feature cimport *
from FeatureMap cimport *
from String cimport *
from TransformationDescription cimport *

cdef extern from "<OpenMS/FORMAT/TransformationXMLFile.h>" namespace "OpenMS":

    cdef cppclass TransformationXMLFile:
        TransformationXMLFile() nogil except +

        void load(String, TransformationDescription &, bool fit_model) nogil except+
        void store(String, TransformationDescription) nogil except+




from libcpp.vector cimport vector as libcpp_vector
from libcpp cimport bool

from MetaInfoInterface cimport *
from IdentificationHit cimport *

cdef extern from "<OpenMS/METADATA/SpectrumIdentification.h>" namespace "OpenMS":

    cdef cppclass SpectrumIdentification(MetaInfoInterface):
        # wrap-inherits:
        #   MetaInfoInterface

        SpectrumIdentification()   nogil except +
        SpectrumIdentification(SpectrumIdentification) nogil except + # wrap-ignore

        # /// sets the identification hits of this spectrum identification (corresponds to single peptide hit in the list)
        void setHits(libcpp_vector[IdentificationHit] & hits) nogil except +

        # /// adds a single identification hit to the hits
        void addHit(IdentificationHit & hit) nogil except +

        # /// returns the identification hits of this spectrum identification
        libcpp_vector[IdentificationHit] getHits() nogil except +




from Types cimport *
from MassTrace cimport *
from Feature cimport *
from FeatureMap cimport *
from MzTab cimport *
from DefaultParamHandler cimport *
from String cimport *
from ProgressLogger cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/AccurateMassSearchEngine.h>" namespace "OpenMS":

    cdef cppclass AccurateMassSearchResult "OpenMS::AccurateMassSearchResult":
        AccurateMassSearchResult() nogil except +
        AccurateMassSearchResult(AccurateMassSearchResult) nogil except +
        double getObservedMZ() nogil except +
        void setObservedMZ(double & m) nogil except +
        double getCalculatedMZ() nogil except +
        void setCalculatedMZ(double & m) nogil except +
        double getQueryMass() nogil except +
        void setQueryMass(double & m) nogil except +
        double getFoundMass() nogil except +
        void setFoundMass(double & m) nogil except +
        double getCharge() nogil except +
        void setCharge(double & ch) nogil except +
        double getMZErrorPPM() nogil except +
        void setMZErrorPPM(double & ppm) nogil except +
        double getObservedRT() nogil except +
        void setObservedRT(double & rt) nogil except +
        double getObservedIntensity() nogil except +
        void setObservedIntensity(double & intensity) nogil except +
        double getMatchingIndex() nogil except +
        void setMatchingIndex(double & idx) nogil except +
        String getFoundAdduct() nogil except +
        void setFoundAdduct(const String & add) nogil except +
        String getFormulaString() nogil except +
        void setEmpiricalFormula(const String & ep) nogil except +
        libcpp_vector[ String ] getMatchingHMDBids() nogil except +
        void setMatchingHMDBids(libcpp_vector[ String ] & match_ids) nogil except +
        double getIsotopesSimScore() nogil except +
        void setIsotopesSimScore(double & sim_score) nogil except +
        libcpp_vector[double] getIndividualIntensities() nogil except +
        void setIndividualIntensities(libcpp_vector[double]) nogil except +
        Size getSourceFeatureIndex() nogil except +
        void setSourceFeatureIndex(Size) nogil except +
        # errors when returning references to vector
        libcpp_vector[ double] getMasstraceIntensities() nogil except +
        void setMasstraceIntensities(libcpp_vector[ double ] & ) nogil except +
